package be.iminds.ilabt.jfed.ui.javafx.automated_testing_gui;

import be.iminds.ilabt.jfed.BaseGuiModule;
import be.iminds.ilabt.jfed.highlevel.HighLevelModule;
import be.iminds.ilabt.jfed.highlevel.LowLevelModule;
import be.iminds.ilabt.jfed.highlevel.UserLoginModelModule;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.ssh_terminal_tool.unix.SshAgentHelper;
import be.iminds.ilabt.jfed.ui.javafx.userlogin.UserLoginFactory;
import com.cathive.fx.guice.GuiceApplication;
import com.cathive.fx.guice.GuiceFXMLLoader;
import com.google.inject.Module;
import javafx.application.Platform;
import javafx.scene.Scene;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import java.io.IOException;
import java.util.List;

public class AutomatedTesterStandalone extends GuiceApplication {
    private static final Logger LOG = LoggerFactory.getLogger(AutomatedTesterStandalone.class);
    private static final String AUTOMATED_TESTER_FXML = "AutomatedTester.fxml";


    @Inject
    private UserLoginFactory userLoginFactory;

    @Inject
    private GuiceFXMLLoader loader;

    @Inject
    private TaskThread tt;

    @Override
    public void start(final Stage stage) throws Exception {
        userLoginFactory.showUserLogin(false, stage, successfulLogin -> showAutomatedTester(stage));
    }

    /**
     * The main() method is ignored in correctly deployed JavaFX application.
     * main() serves only as fallback in case the application can not be
     * launched through deployment artifacts, e.g., in IDEs with limited FX
     * support. NetBeans ignores main().
     *
     * @param args the command line arguments
     */
    public static boolean debugEnabled = false;

    public static void main(String[] args) {
        for (String arg : args) {
            if (arg.equalsIgnoreCase("debug") || arg.equalsIgnoreCase("--debug") || arg.equalsIgnoreCase("-debug"))
                debugEnabled = true;
        }
        launch(args);
    }

    @Override
    public void init(List<Module> modules) throws Exception {
        modules.add(new LowLevelModule());
        modules.add(new HighLevelModule());
        modules.add(new UserLoginModelModule());
        modules.add(new BaseGuiModule());
    }

    private void showAutomatedTester(Stage stage) {
        stage.setTitle("jFed Automated Tester");


        try {
            GuiceFXMLLoader.Result result =
                    loader.load(AutomatedTesterStandalone.class.getResource(AUTOMATED_TESTER_FXML));

            stage.setScene(new Scene(result.getRoot()));

            stage.setOnCloseRequest(event -> exit());

            stage.show();
        } catch (IOException e) {
            LOG.error("Error while loading jFed Automated Tester", e);
        }

    }

    public void exit() {
        LOG.info("Close was requested. Trying to terminate");
        tt.requestStop();

        //stop any running ssh-agent
        SshAgentHelper.stopSshAgent();

        Platform.exit();
        System.exit(0);
    }

}
