package be.iminds.ilabt.jfed.preferences;

import be.iminds.ilabt.jfed.util.OSDetector;

/**
* GuiPreferenceKey
*/
public enum GuiPreferenceKey implements JFedPreferences.PreferenceKey {
    /**
     * Command to start a Unix/Mac terminal
     */
    PREF_COMMAND_UNIX_TERMINAL("command.unix-terminal", OSDetector.getUnixLikeOsFilter()),
    /**
     * The SSH command WITHOUT proxy to be used in a terminal
     */
    PREF_COMMAND_SSH("command.ssh", OSDetector.getUnixLikeOsFilter()),
    /**
     * The SSH command WITH proxy to be used in a terminal
     */
    PREF_COMMAND_SSH_PROXY("command.ssh-proxy", OSDetector.getUnixLikeOsFilter()),
    /**
     * Choose SSH command automatically
     */
    PREF_COMMAND_SSH_AUTO("command.ssh-proxy-auto"),

    /**
     * Directory of PuTTY-installation
     */
    PREF_PUTTY_DIRECTORY("putty.dir", new OSDetector.OneOsFilter(OSDetector.OS.WIN)),
    /**
     * Whether Pageant must be activated (Windows only, use @link{PREF_SSHAGENT_USE} otherwise)
     */
    PREF_PUTTY_PAGEANT("putty.usepageant", new OSDetector.OneOsFilter(OSDetector.OS.WIN)),
    /**
     * Whether X11-forwarding on windows must be activated (Windows only)
     */
    PREF_PUTTY_X11_FORWARD("putty.x11forward", new OSDetector.OneOsFilter(OSDetector.OS.WIN)),

    /**
     * Is SSH-agent enabled? (Unix/Mac only, use @link{PREF_PUTTY_PAGEANT} for Windows)
     */
    PREF_SSHAGENT_USE("sshagent.use", OSDetector.getUnixLikeOsFilter()),
    PREF_SHOW_MAC_TERMINAL_WARNING("show-mac-terminal-warning"),

    PREF_TERMINAL_DEBUG("terminal.debug"),
    PREF_TERMINAL_KEEP_TMP_FILES("terminal.keep-tmp-files"),

    PREF_SCS_URN("scs.urn"),
    PREF_SCS_URL("scs.url"),

    //SSH keys. If not specified, keys from "user login file" (that are also used for SSL) are used.
    //These keys are only used for SSH, never for SSL (login file is used for that)
    //on windows, the SSH keypair needs to be in putty format! (ppk)
    PREF_SSH_OVERRIDE("sshkeys.override"),
    PREF_SSH_PRIVATE_KEY_FILE("sshkeys.private_key.file"),
    PREF_SSH_PUBLIC_KEY_FILE("sshkeys.public_key.file"),
    PREF_SSH_PUTTY_KEY_FILE("sshkeys.putty_key.file"), //putty key file takes precedence over other files and text, on windows
    PREF_SSH_PRIVATE_KEY("sshkeys.private_key.text"),
    PREF_SSH_PUBLIC_KEY("sshkeys.public_key.text"),

    //override SSH proxy settings. private key MUST be unencrypted. On windows it needs to be PuTTY key file.
    PREF_SSHPROXY_OVERRIDE("sshproxy.override"),
    PREF_SSHPROXY_HOSTNAME("sshproxy.hostname"),
    PREF_SSHPROXY_PORT("sshproxy.port"),
    PREF_SSHPROXY_USERNAME("sshproxy.username"),
    PREF_SSHPROXY_HOSTKEY("sshproxy.hostkey"),
    PREF_SSHPROXY_PUBLIC_KEY_FILE("sshproxy.public_key.file"),
    PREF_SSHPROXY_PRIVATE_KEY_FILE("sshproxy.private_key.file"),
    PREF_SSHPROXY_PUTTY_KEY_FILE("sshproxy.putty_key.file"),
    PREF_SSHPROXY_PUBLIC_KEY("sshproxy.public_key.text"),
    PREF_SSHPROXY_PRIVATE_KEY("sshproxy.private_key.text"),

    PREF_SSHPROXY_USE_FOR_JFED("sshproxy.use_for_jfed"), //AUTO, ALWAYS or NEVER
    PREF_SSHPROXY_USE_FOR_SSH("sshproxy.use_for_ssh"), //AUTO, ALWAYS or NEVER

    PREF_FIRSTRUN_VERSION("firstrunversion"), //a number.

    PREF_RSPECFOLDER_REQUEST("files.rspec_folder"),
    PREF_RSPECFOLDER_MANIFEST("files.manifest_rspec_folder"),

    PREF_SHOW_SLICE_INITIALIZATION_DIALOG("slice.show_init_dialog"),
    PREF_SHOW_CONNECTIVITY_WARNING_DIALOG("slice.show_connectivity_warning_dialog"),

    PREF_IS_DEMO("is_demo");

    final String key;
    final OSDetector.OSFilter osFilter;

    GuiPreferenceKey(String key) {
        this.key = key;
        this.osFilter = null;
    }

    GuiPreferenceKey(String key, OSDetector.OSFilter osFilter) {
        this.key = key;
        this.osFilter = osFilter;
    }

    @Override
    public String toString() {
        return key;
    }

    @Override
    public String getKey() {
        return key;
    }

    @Override
    public boolean isOSSpecific() {
        return osFilter != null;
    }

    @Override
    public OSDetector.OSFilter getOSFilter() {
        return osFilter;
    }
}
