package be.iminds.ilabt.jfed.ui.javafx.log_gui;

import be.iminds.ilabt.jfed.call_log_output.HtmlLogOutput;
import be.iminds.ilabt.jfed.call_log_output.SerializableApiCallDetails;
import be.iminds.ilabt.jfed.ui.javafx.util.TimeUtils;
import be.iminds.ilabt.jfed.util.IOUtils;
import be.iminds.ilabt.jfed.util.TextUtil;
import be.iminds.ilabt.jfed.util.XmlUtil;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.value.ChangeListener;
import javafx.concurrent.Worker;
import javafx.fxml.FXML;
import javafx.scene.Parent;
import javafx.scene.control.*;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;
import javafx.stage.FileChooser;
import org.controlsfx.glyphfont.FontAwesome;
import org.controlsfx.glyphfont.GlyphFont;
import org.controlsfx.glyphfont.GlyphFontRegistry;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.File;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class LogPanel {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(LogPanel.class);
    private static final GlyphFont fontAwesome = GlyphFontRegistry.font("FontAwesome");
    private final ObjectProperty<SerializableApiCallDetails> shownLog = new SimpleObjectProperty<>();
    @FXML
    private TextArea resultText;
    @FXML
    private TextArea resultErrorText;
    @FXML
    private TextField authorityNameText;
    @FXML
    private VBox resultTabContent;
    private boolean showingResultErrorText = true;
    private boolean showingResultText = true;
    @FXML
    private Parent resultTextGroup;
    @FXML
    private Parent resultErrorTextGroup;
    @FXML
    private Label resultCode;
    @FXML
    private Label resultLabel;
    @FXML
    private TextArea lowLevelGeniValueText;
    @FXML
    private TextArea lowLevelGeniCodeText;
    @FXML
    private TextArea lowLevelGeniOutputText;
    @FXML
    private TextField xmlRpcCommand;
    @FXML
    private TextArea xmlRpcSentText;
    @FXML
    private TextArea xmlRpcReceivedText;
    @FXML
    private StackPane xmlRpcSentStackPane;
    @FXML
    private StackPane xmlRpcReplyStackPane;
    @FXML
    private Button xmlRpcSentSwitchViewButton;
    @FXML
    private Button xmlRpcReplySwitchViewButton;
    @FXML
    private XmlRpcPane xmlRpcSentPane;
    @FXML
    private XmlRpcPane xmlRpcReceivedPane;
    @FXML
    private TextField serverUrlText;
    @FXML
    private TextField proxyText;
    @FXML
    private TextField connectionDebugIdText;
    @FXML
    private TextField connectionUserUrnText;
    @FXML
    private TextField httpRequestLineText;
    @FXML
    private TextArea httpSentHeaders;
    @FXML
    private TextArea httpSentText;
    @FXML
    private TextField httpStatusLineText;
    @FXML
    private TextArea httpReceivedHeaders;
    @FXML
    private TextArea httpReceivedText;
    @FXML
    private TabPane logTabPane;
    @FXML
    private VBox logPanelContainer;
    @FXML
    private Tab resultTab;
    @FXML
    private Tab lowLevelValueTab;
    @FXML
    private Tab lowLevelOutputTab;
    //    @FXML private Tab XmlRpcTab;
//    @FXML private Tab httpTab;
    @FXML
    private Tab XmlRpcRequestTab;
    @FXML
    private Tab XmlRpcReplyTab;
    @FXML
    private Tab connectionDetailsTab;
    @FXML
    private Tab httpRequestTab;
    @FXML
    private Tab httpReplyTab;
    //extra tabs
    @FXML
    private Tab stitchingOverviewTab;
    @FXML
    private Tab exceptionTab;
    @FXML
    private Tab xmlTab;
    @FXML
    private Tab spewLogTab;
    @FXML
    private Tab htmlErrorTab;
    @FXML
    private Label requestSizeLabel;
    @FXML
    private Label replySizeLabel;
    @FXML
    private Label requestSizeHeaderLabel;
    @FXML
    private Label replySizeHeaderLabel;
    @FXML
    private TextArea exceptionTextField;
    @FXML
    private TextArea stitchingOverviewTextArea;
    @FXML
    private TextField callStartTimeField;
    @FXML
    private Label callStartTimeRelativeLabel;
    @FXML
    private TextField callStopTimeField;
    @FXML
    private Label callStopTimeRelativeLabel;
    @FXML
    private TextArea xmlText;
    private String xmlToFormat;
    private String shownExceptionText = null;
    @FXML
    private WebView htmlWebView;
    private SerializableApiCallDetails htmlApiCallDetails;
    @FXML
    private WebView spewWebView;
    @FXML
    private Label spewWebViewLabel;
    @FXML
    private TextField spewUrl;
    @FXML
    private TextField spewUrn;
    private ObjectProperty<SerializableApiCallDetails> shownApiCallDetails = new SimpleObjectProperty<>(null);
    private final ChangeListener<SerializableApiCallDetails> shownLogChangeListener =
            (observableValue, oldApiCallDetails, newApiCallDetails) -> {
                //no need to bind! ApiCallDetails never changes...
                logTabPane.getTabs().removeAll(xmlTab, spewLogTab, htmlErrorTab, exceptionTab, stitchingOverviewTab);

                shownApiCallDetails.set(newApiCallDetails);

                if (newApiCallDetails == null) {
                    resultText.setText("");
                    lowLevelGeniValueText.setText("");
                    lowLevelGeniOutputText.setText("");
                    lowLevelGeniCodeText.setText("");
                    xmlRpcCommand.setText("");
                    xmlRpcSentText.setText("");
                    xmlRpcReceivedText.setText("");
                    xmlRpcSentPane.setObj(null);
                    xmlRpcReceivedPane.setObj(null);
                    authorityNameText.setText("");
                    serverUrlText.setText("");
                    proxyText.setText("");
                    connectionDebugIdText.setText("");
                    connectionUserUrnText.setText("");
                    callStartTimeField.setText("");
                    callStartTimeRelativeLabel.setText("");
                    callStopTimeField.setText("");
                    callStopTimeRelativeLabel.setText("");
                    requestSizeLabel.setVisible(false);
                    replySizeLabel.setVisible(false);
                    httpRequestLineText.setText("");
                    httpSentHeaders.setText("");
                    httpSentText.setText("");
                    httpStatusLineText.setText("");
                    httpReceivedHeaders.setText("");
                    httpReceivedText.setText("");
                    xmlText.setText("");

                    showResultErrorText(false);
                    showResultText(false);

                    resultTab.setDisable(true);
                    lowLevelValueTab.setDisable(true);
                    lowLevelOutputTab.setDisable(true);
                    XmlRpcRequestTab.setDisable(true);
                    XmlRpcReplyTab.setDisable(true);
                    connectionDetailsTab.setDisable(true);
                    httpRequestTab.setDisable(true);
                    httpReplyTab.setDisable(true);
                } else {
                    resultTab.setDisable(false);
                    lowLevelValueTab.setDisable(false);
                    lowLevelOutputTab.setDisable(false);
                    XmlRpcRequestTab.setDisable(false);
                    XmlRpcReplyTab.setDisable(false);
                    connectionDetailsTab.setDisable(false);
                    httpRequestTab.setDisable(false);
                    httpReplyTab.setDisable(false);

                    if (newApiCallDetails.getAuthorityUrn() != null)
                        authorityNameText.setText(newApiCallDetails.getAuthorityUrn());
                    else
                        authorityNameText.setText("");
                    serverUrlText.setText(newApiCallDetails.getServerUrl());
                    proxyText.setText(newApiCallDetails.getProxyInfo() == null ? "no proxy used" : newApiCallDetails.getProxyInfo().toString());
                    connectionUserUrnText.setText(newApiCallDetails.getConnectionUserUrn());
                    connectionDebugIdText.setText(newApiCallDetails.getConnectionId());

                    Date startTime = newApiCallDetails.getStartTime();
                    Date stopTime = newApiCallDetails.getStopTime();
                    if (startTime == null) {
                        callStartTimeField.setText("");
                        callStartTimeRelativeLabel.setText("");
                    } else {
                        callStartTimeField.setText("" + startTime);
                        long millis = System.currentTimeMillis() - startTime.getTime();
                        String relString = TimeUtils.formatMillis(millis, TimeUnit.DAYS, TimeUnit.SECONDS);
                        callStartTimeRelativeLabel.setText("(" + relString + " ago)");
                    }
                    if (stopTime == null) {
                        callStopTimeField.setText("");
                        callStopTimeRelativeLabel.setText("");
                    } else {
                        callStopTimeField.setText("" + stopTime);
                        String res = "(";
                        long millis = System.currentTimeMillis() - stopTime.getTime();
                        res += TimeUtils.formatMillis(millis, TimeUnit.DAYS, TimeUnit.SECONDS) + " ago";
                        if (startTime != null) {
                            long millis2 = stopTime.getTime() - startTime.getTime();
                            res +=
                                    " = " + TimeUtils.formatMillis(millis2, TimeUnit.DAYS, TimeUnit.MILLISECONDS) + " after start";
                        }
                        res += ")";
                        callStopTimeRelativeLabel.setText(res);
                    }

                    httpRequestLineText.setText(
                            newApiCallDetails.getHttpRequestStatusLine() == null ? "<no data>"
                                    : newApiCallDetails.getHttpRequestStatusLine()
                    );
                    httpSentHeaders.setText(
                            newApiCallDetails.getHttpRequestHeaders() == null ? "<no data>"
                                    : newApiCallDetails.getHttpRequestHeaders()
                    );
                    httpSentText.setText(
                            newApiCallDetails.getHttpRequest() == null ? "<no data>"
                                    : newApiCallDetails.getHttpRequest()
                    );
                    httpStatusLineText.setText(
                            newApiCallDetails.getHttpResponseStatusLine() == null ? "<no data>"
                                    : newApiCallDetails.getHttpResponseStatusLine()
                    );
                    httpReceivedHeaders.setText(
                            newApiCallDetails.getHttpResponseHeaders() == null ? "<no data>"
                                    : newApiCallDetails.getHttpResponseHeaders()
                    );
                    httpReceivedText.setText(
                            newApiCallDetails.getHttpResponse() == null ? "<no data>"
                                    : newApiCallDetails.getHttpResponse()
                    );

                    requestSizeLabel.setVisible(true);
                    replySizeLabel.setVisible(true);
                    long requestSizeByte = 0;
                    if (newApiCallDetails.getHttpRequestStatusLine() != null)
                        requestSizeByte += newApiCallDetails.getHttpRequestStatusLine().length();
                    if (newApiCallDetails.getHttpRequestHeaders() != null)
                        requestSizeByte += newApiCallDetails.getHttpRequestHeaders().length();
                    if (newApiCallDetails.getHttpRequest() != null)
                        requestSizeByte += newApiCallDetails.getHttpRequest().length();
                    requestSizeLabel.setText(requestSizeByte + "");
                    long replySizeByte = 0;
                    if (newApiCallDetails.getHttpResponseStatusLine() != null)
                        replySizeByte += newApiCallDetails.getHttpResponseStatusLine().length();
                    if (newApiCallDetails.getHttpResponseHeaders() != null)
                        replySizeByte += newApiCallDetails.getHttpResponseHeaders().length();
                    if (newApiCallDetails.getHttpResponse() != null)
                        replySizeByte += newApiCallDetails.getHttpResponse().length();
                    replySizeLabel.setText(replySizeByte + "");

                    xmlRpcCommand.setText(newApiCallDetails.getGeniMethodName());
                    String xmlRpcRequestHumanReadableString =
                            newApiCallDetails.getXmlRpcRequestHumanReadableString();
                    xmlRpcSentText.setText(xmlRpcRequestHumanReadableString);
                    String xmlRpcResponseHumanReadableString =
                            newApiCallDetails.getXmlRpcResponseHumanReadableString();
                    xmlRpcReceivedText.setText(xmlRpcResponseHumanReadableString);

                    if (xmlRpcRequestHumanReadableString.length() < 100000)
                        xmlRpcSentPane.setObj(newApiCallDetails.getXmlRpcRequestJsonObject());
                    else
                        xmlRpcSentPane.setObj("output too long for this view");
                    if (xmlRpcResponseHumanReadableString.length() < 100000)
                        xmlRpcReceivedPane.setObj(newApiCallDetails.getXmlRpcResponseJsonObject());
                    else
                        xmlRpcReceivedPane.setObj("output too long for this view");

                    if (isHtml(newApiCallDetails.getHttpResponse()))
                        showHtml(newApiCallDetails);

                    //result code and error output (error output only hidden if there is none and the call was successful)
                    if (newApiCallDetails.getGeniResponseCode() != null) {
                        resultCode.setText("GeniResponseCode=" + newApiCallDetails.getGeniResponseCode() + " = " + newApiCallDetails.getGeniResponseCodeDescription());

                        assert (newApiCallDetails.getGeniResponseCodeIsSuccess() != null);
                        if (newApiCallDetails.getGeniResponseCodeIsSuccess() &&
                                (newApiCallDetails.getGeniResponseOutput() == null ||
                                        newApiCallDetails.getGeniResponseOutput().isEmpty())) {
                            resultErrorText.setText("");
                            showResultErrorText(false);
                        } else {
                            resultErrorText.setText(newApiCallDetails.getGeniResponseOutput());
                            showResultErrorText(true);
                        }

                    } else {
                        resultCode.setText("ERROR: no GeniResponseCode");
                        showResultErrorText(true);
                    }


                    if (newApiCallDetails.getRawValue() != null) {
                        resultTab.setDisable(false);

                        showResultText(true);
                        resultText.setText(newApiCallDetails.getRawValue());
                        resultText.setDisable(false);

                        //very broad detection
                        if (newApiCallDetails.getRawValue().contains("stitching"))
                            showStitchingOverview(newApiCallDetails);


                        //determine if XML-tab should be shown
                        if (isXml(newApiCallDetails.getRawValue())) {
                            showXml(newApiCallDetails.getRawValue());
                            //show Xml by looking at low level reply, so even when reinterpreted
                            // (for example as AnyCredential). Does not show compressed Xml.
                        } else if (newApiCallDetails.getXmlRpcGeniResponseValue() != null
                                && isXml(newApiCallDetails.getXmlRpcGeniResponseValue())) {
                            showXml(newApiCallDetails.getXmlRpcGeniResponseValue());
                        }

                        lowLevelValueTab.setDisable(false);
                        lowLevelOutputTab.setDisable(false);

                        if (newApiCallDetails.getXmlRpcGeniResponseValue() != null) {
                            lowLevelGeniValueText.setDisable(false);
                            lowLevelGeniValueText.setText(newApiCallDetails.getXmlRpcGeniResponseValue());
                        } else {
                            lowLevelGeniValueText.setDisable(true);
                            lowLevelGeniValueText.setText("ERROR: raw result is a hashtable without \"value\" field:\n\n"
                                    + newApiCallDetails.getRawValue());
                        }

                        if (newApiCallDetails.getXmlRpcGeniResponseCode() != null) {
                            lowLevelGeniCodeText.setDisable(false);
                            lowLevelGeniCodeText.setText(newApiCallDetails.getXmlRpcGeniResponseCode());
                        } else {
                            lowLevelGeniCodeText.setDisable(true);
                            lowLevelGeniOutputText.setText("no \"code\" field in result hashtable");
                        }

                        if (newApiCallDetails.getXmlRpcGeniResponseOutput() != null) {
                            lowLevelGeniOutputText.setDisable(false);
                            lowLevelGeniOutputText.setText(newApiCallDetails.getXmlRpcGeniResponseOutput());
                        } else {
                            lowLevelGeniOutputText.setDisable(true);
                            lowLevelGeniOutputText.setText("no \"output\" field in result hashtable");
                        }

                    } else {

                        //very broad detection
                        if (newApiCallDetails.getHttpRequest().contains("stitching") ||
                                newApiCallDetails.getHttpResponse().contains("stitching"))
                            showStitchingOverview(newApiCallDetails);

                        resultCode.setText("ERROR: no reply");
                        showResultErrorText(false);

                        resultText.setText("<null>");
                        showResultText(false);

                        resultTab.setDisable(true);
                        lowLevelValueTab.setDisable(true);
                        lowLevelOutputTab.setDisable(true);
                        lowLevelGeniValueText.setText("ERROR: no reply");
                        lowLevelGeniOutputText.setText("");
                        lowLevelGeniCodeText.setText("");
                        lowLevelGeniValueText.setDisable(true);
                        lowLevelGeniOutputText.setDisable(true);
                        lowLevelGeniCodeText.setDisable(true);
                    }


                    if (newApiCallDetails.getProtogeniSpewLogUrl() != null)
                        showSpewLogFile(newApiCallDetails.getProtogeniSpewLogUrl());


                    if (newApiCallDetails.getExceptionString() != null)
                        showExceptionString(newApiCallDetails.getExceptionString());
                }
            };

//    private static Element quickDomChildHelper(Element e, String tagName) {
//        NodeList resList = e.getElementsByTagName(tagName);
//        assert resList.getLength() == 1;
//        org.w3c.dom.Node resNode = resList.item(0);
//        assert resNode != null;
//        assert resNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE;
//        return (Element) resNode;
//    }
//
//    private static List<Element> quickDomChildrenHelper(Element e, String tagName) {
//        NodeList resList = e.getElementsByTagName(tagName);
//        List<Element> resElements = new ArrayList<>();
//        for (int i = 0; i < resList.getLength(); i++) {
//            org.w3c.dom.Node resNode = resList.item(i);
//            assert resNode != null;
//            assert resNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE;
//            Element resElement = (Element) resNode;
//            resElements.add(resElement);
//        }
//        return resElements;
//    }

    public ObjectProperty<SerializableApiCallDetails> shownLogProperty() {
        return shownLog;
    }

    public SerializableApiCallDetails getShownLog() {
        return shownLog.get();
    }

    public void setShownLog(SerializableApiCallDetails details) {
        shownLog.set(details);
    }

    @FXML
    private void initialize() {
        shownLog.addListener(shownLogChangeListener);

//        rspecShowPanel.editableProperty().set(false);
        logTabPane.getTabs().removeAll(stitchingOverviewTab, xmlTab, spewLogTab, htmlErrorTab, exceptionTab);
        logTabPane.getSelectionModel().select(resultTab);

        requestSizeLabel.managedProperty().bind(requestSizeLabel.visibleProperty());
        replySizeLabel.managedProperty().bind(replySizeLabel.visibleProperty());
        requestSizeHeaderLabel.managedProperty().bind(requestSizeHeaderLabel.visibleProperty());
        replySizeHeaderLabel.managedProperty().bind(replySizeHeaderLabel.visibleProperty());
        requestSizeHeaderLabel.visibleProperty().bind(requestSizeLabel.visibleProperty());
        replySizeHeaderLabel.visibleProperty().bind(replySizeLabel.visibleProperty());

        xmlRpcReplySwitchViewButton.setGraphic(fontAwesome.create(FontAwesome.Glyph.EYE).color(Color.BLUE));
        for (javafx.scene.Node node : xmlRpcReplyStackPane.getChildren()) {
            node.managedProperty().bind(node.visibleProperty());
        }
        xmlRpcSentSwitchViewButton.setGraphic(fontAwesome.create(FontAwesome.Glyph.EYE).color(Color.BLUE));
        for (javafx.scene.Node node : xmlRpcSentStackPane.getChildren()) {
            node.managedProperty().bind(node.visibleProperty());
        }
    }

    public void onXmpRpcSentSwitchView() {
        switchStackPaneVisibleItem(xmlRpcSentStackPane);
    }

    public void onXmpRpcReplySwitchView() {
        switchStackPaneVisibleItem(xmlRpcReplyStackPane);
    }

    public void switchStackPaneVisibleItem(StackPane sp) {
        int nextVisible = 0;
        List<javafx.scene.Node> nodes = sp.getChildren();
        for (int i = 0; i < nodes.size(); i++) {
            javafx.scene.Node node = nodes.get(i);
            if (node.isVisible()) {
                nextVisible = i + 1;
                node.setVisible(false);
            }
        }
        if (nextVisible >= nodes.size())
            nextVisible = 0;
        if (nextVisible < nodes.size())
            nodes.get(nextVisible).setVisible(true);
    }

    private void showResultText(boolean show) {
        if (showingResultText == show) return;
        showingResultText = show;
        if (!show)
            resultTabContent.getChildren().remove(resultTextGroup);
        else
            resultTabContent.getChildren().add(resultTextGroup);
    }

    private void showResultErrorText(boolean show) {
        if (showingResultErrorText == show) return;
        showingResultErrorText = show;
        if (!show)
            resultTabContent.getChildren().remove(resultErrorTextGroup);
        else
            resultTabContent.getChildren().add(resultErrorTextGroup);
    }

    public boolean isHtml(String text) { //TODO: better test
        if (text == null) return false;
        String head = text;
        if (text.length() > 100)
            head = text.substring(0, 100);
        return head.contains("<html") || head.contains("<HTML") || head.contains("<!DOCTYPE HTML");
    }

    public boolean isXml(String text) { //TODO: better test
        if (text == null) return false;
        String head = text;
        if (text.length() > 100)
            head = text.substring(0, 100);
        return head.contains("<?xml") || head.contains("<rspec");
    }

    public boolean isRspec(String text) { //TODO: better test
        if (text == null) return false;
        String head = text;
        if (text.length() > 100)
            head = text.substring(0, 100);
        return head.contains("<rspec");
    }

    public void showXml(String text) {
        this.xmlToFormat = text;
        xmlText.setText("");
        logTabPane.getTabs().add(xmlTab);
        repairTabPane();
    }

    @FXML
    private void showXmlTextLog() {
        if (xmlToFormat != null) {
            try {
                String formattedXml = XmlUtil.formatXmlFromString(xmlToFormat);
                xmlText.setText(formattedXml);
                xmlToFormat = null;
            } catch (Exception e) {
                LOG.warn("Exception while formatting XML, original xml: \"" + xmlToFormat + "\"", e);
                xmlText.setText("Exception while formatting XML:\n\n" + TextUtil.exceptionToString(e) + "\n\nOriginal xml:\n"+xmlToFormat+"\n");
            }
        }
    }



    public void showStitchingOverview(SerializableApiCallDetails callDetails) {
        if (callDetails != null) {
            logTabPane.getTabs().add(stitchingOverviewTab);
            repairTabPane();

            String stitchingOverviewText = HtmlLogOutput.makeStitchingOverview(callDetails);

            assert stitchingOverviewTextArea != null;
            stitchingOverviewTextArea.setText(stitchingOverviewText+"");
        }
    }

    public void showExceptionString(String exceptionStackTrace) {
        if (exceptionStackTrace != null) {
            logTabPane.getTabs().add(exceptionTab);
            repairTabPane();
            String exceptionText = exceptionStackTrace + "\n";
            shownExceptionText = exceptionText;

            assert exceptionTextField != null;
            exceptionTextField.setText(exceptionText);
        }
    }

    @FXML
    private void printException() {
        if (shownExceptionText != null)
            System.err.println(shownExceptionText);
    }

    public void showHtml(SerializableApiCallDetails apiCallDetails) {
        this.htmlApiCallDetails = apiCallDetails;
        logTabPane.getTabs().add(htmlErrorTab);
        repairTabPane();
//        htmlErrorTab.setDisable(false);
    }

    @FXML
    private void showHtmlErrorLog() {
        if (htmlApiCallDetails != null) {
            final WebEngine webEngine = htmlWebView.getEngine();
            webEngine.loadContent(htmlApiCallDetails.getHttpResponse());
            htmlApiCallDetails = null;
        }
    }

    public void showSpewLogFile(String url) {
        spewUrl.setText(url);
        logTabPane.getTabs().add(spewLogTab);
        repairTabPane();

        showSpewLog();
    }

    public void showSpewLog() {
        final WebEngine webEngine = spewWebView.getEngine();
        LOG.trace("Loading URL into spewWebView: \"" + spewUrl.getText() + "\"");

        webEngine.getLoadWorker().stateProperty().addListener(
                (ov, oldState, newState) -> {
                    spewWebViewLabel.setText("ProtoGeni Error Log (HTML loading status: " + newState + "):");
                    if (newState == Worker.State.FAILED) {
                        if (spewUrl.getText().startsWith("https"))
                            spewWebViewLabel.setText("ProtoGeni Error Log Failed to load. You are using SSL, perhaps the certificate is self-signed? (This is not yet supported when showing logs)");
                        else
                            spewWebViewLabel.setText("ProtoGeni Error Log Failed to load.");
                    }
                }
        );
        webEngine.setOnAlert(
                msg -> LOG.trace("Spew Log onAlert: " + msg)
        );
        webEngine.setOnStatusChanged(
                msg -> LOG.trace("Spew Log onStatusChanged: " + msg)
        );

        webEngine.load(spewUrl.getText());
    }

    private String getSpew() {
        final WebEngine webEngine = spewWebView.getEngine();
        Document doc = webEngine.getDocument();
        try {
            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
            transformer.setOutputProperty(OutputKeys.METHOD, "xml");
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");

            StringWriter sw = new StringWriter();
            transformer.transform(new DOMSource(doc), new StreamResult(sw));
            return sw.getBuffer().toString();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

    @FXML
    private void saveSpew() {
        askSave(getSpew(), "txt");
    }

    @FXML
    private void saveXml() {
        askSave(xmlText.getText(), "xml");
    }

    @FXML
    private void saveResultText() {
        askSave(resultText.getText(), "txt");
    }

    @FXML
    private void saveResultError() {
        askSave(resultErrorText.getText(), "txt");
    }

    @FXML
    private void saveLowLevel() {
        askSave(lowLevelGeniValueText.getText(), "txt");
    }

    @FXML
    private void saveXmlRpcSent() {
        askSave(xmlRpcSentText.getText(), "txt");
    }

    @FXML
    private void saveXmlRpcRecv() {
        askSave(xmlRpcReceivedText.getText(), "txt");
    }

    @FXML
    private void saveHttpSent() {
        askSave(httpSentText.getText(), "txt");
    }

    @FXML
    private void saveHttpRecv() {
        askSave(httpReceivedText.getText(), "txt");
    }

    @FXML
    private void saveAll() {
        SerializableApiCallDetails apiCallDetails = shownApiCallDetails.get();
        String text =
                "Call \"" + apiCallDetails.getGeniMethodName() + "\" in API \"" + apiCallDetails.getApiName() + "\"\n";
        text +=
                "Authority: \"" + apiCallDetails.getAuthorityName() + "\" urn=" + apiCallDetails.getAuthorityUrn() + "\n";
        text += "Server URL: \"" + apiCallDetails.getServerUrl() + "\"\n\n";
        text += "Proxy: \"" + apiCallDetails.getProxyInfo() + "\"\n\n";
        text += "Start time: \"" + apiCallDetails.getStartTime() + "\"\n\n";
        text += "Stop time: \"" + apiCallDetails.getStopTime() + "\"\n\n";
        text += "HTTP data sent:\n" + apiCallDetails.getHttpRequest() + "\n\n";
        text += "HTTP data recv:\n" + apiCallDetails.getHttpResponse() + "\n\n";
        text += "XmlRpc data sent:\n" + apiCallDetails.getXmlRpcRequestJsonString() + "\n\n";
        text += "XmlRpc data recv:\n" + apiCallDetails.getXmlRpcResponseJsonString() + "\n\n";

        if (apiCallDetails.getXmlRpcGeniResponseValue() != null) {
            text += "Reply recv:\n" + apiCallDetails.getXmlRpcGeniResponseValue() + "\n\n";
        }

        if (apiCallDetails.getProtogeniSpewLogUrl() != null)
            text += "Spew log:\n" + getSpew() + "\n\n";

        askSave(text, "txt");

    }

    @FXML
    private void saveAllAsXml() {
        SerializableApiCallDetails apiCallDetails = shownApiCallDetails.get();

        StringWriter sw = new StringWriter();
        try {
            Marshaller m = JAXBContext.newInstance(SerializableApiCallDetails.class).createMarshaller();
            m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            m.marshal(apiCallDetails, sw);
        } catch (JAXBException e) {
            e.printStackTrace();
        }

        String xml = sw.getBuffer().toString();

        askSave(xml, "xml");

    }

    private void askSave(String text, String ext) {
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("Save " + ext);
        fileChooser.getExtensionFilters().addAll(
                new FileChooser.ExtensionFilter(ext + " Files", "*." + ext),
                new FileChooser.ExtensionFilter("All Files", "*.*")
        );
        File file = fileChooser.showSaveDialog(/*stage*/null);
        if (file != null) {
            IOUtils.stringToFile(file, text);
        }
    }

    /**
     * temporary solution for bug https://javafx-jira.kenai.com/browse/RT-39339. Should be removed again after release of j8u40
     */
    private void repairTabPane() {
        List<Tab> allTabs = new ArrayList<>(logTabPane.getTabs());
        logTabPane.getTabs().clear();
        TabPane newTabPane = new TabPane();
        newTabPane.getTabs().addAll(allTabs);

        logPanelContainer.getChildren().remove(logPanelContainer.getChildren().size() - 1);
        logPanelContainer.getChildren().add(newTabPane);

        logTabPane = newTabPane;
    }
}
