package be.iminds.ilabt.jfed.experimenter_gui.bugreporting;

import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.event.EventHandler;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseEvent;
import javafx.scene.text.Text;
import javafx.util.Callback;

/**
 * LogLineTable
 */
public class LogLineTable extends TableView<LogLine> {
    private TableColumn levelCol = new TableColumn("Level");
    private TableColumn timeCol = new TableColumn("Time");
    private TableColumn classCol = new TableColumn("Class");
    private TableColumn messageCol = new TableColumn("Message");
    private TableColumn throwableCol = new TableColumn("Exception");
//    private TableColumn threadCol = new TableColumn("Thread");

    public LogLineTable() {
        getStylesheets().add(LogLineTable.class.getResource("LogLineTable.css").toExternalForm());

        setEditable(false);
        final EventHandler<MouseEvent> cellDoubleClickHandler = new EventHandler<MouseEvent>() {
            @Override
            public void handle(MouseEvent event) {
                if (event.getClickCount() > 1) {
                    TableCell c = (TableCell) event.getSource();
                    TableColumn col = c.getTableColumn();
                    String text = c.getText();
                    if (col == throwableCol)
                        System.err.print("\nUser requested print of this stacktrace:\n" + text + "\n");
                    if (col == messageCol) {
                        System.out.println("LOG Message:\n\n" + text);
                        JFDialogs.create()
                                .owner(null)
                                .message("LOG Message:\n\n" + text)
                                .masthead(null)
                                .title(null)
                                .showInformation();
                    }

                    double targetWidth = c.prefWidthProperty().get();
                    if (targetWidth <= 0) {
                        final Text javaFXText = new Text(text);
                        //if css needs to be applied, this would require some more tricks:
//                                        new Scene(new Group(text));
//
//                                            // java 7 =>
//                                            //    text.snapshot(null, null);
//                                            // java 8 =>
//                                            text.applyCss();

                        targetWidth = javaFXText.getLayoutBounds().getWidth();
                    }

                    double maxColSize = 0.6 * LogLineTable.this.widthProperty().get();
                    if (col != throwableCol && targetWidth > maxColSize) {
                        targetWidth = maxColSize;
                    }
                    col.setPrefWidth(targetWidth + 5);
                    col.setMinWidth(targetWidth + 5);
                }
            }
        };

        Callback<TableColumn, TableCell> cellFactory =
                new Callback<TableColumn, TableCell>() {
                    public TableCell call(TableColumn p) {
                        final TableCell cell = new TableCell<LogLine, Object>() {
                            @Override
                            public void updateItem(Object item, boolean empty) {
                                super.updateItem(item, empty);
                                setText(empty ? null : getString());
                                setGraphic(null);
                                setWrapText(true);
                            }

                            private String getString() {
                                return getItem() == null ? "" : getItem().toString();
                            }
                        };

                        cell.addEventFilter(MouseEvent.MOUSE_CLICKED, cellDoubleClickHandler);
                        return cell;
                    }
                };
        Callback<TableColumn, TableCell> levelCellFactory =
                new Callback<TableColumn, TableCell>() {
                    public TableCell call(TableColumn p) {
                        final TableCell cell = new TableCell<LogLine, Object>() {
                            @Override
                            public void updateItem(Object item, boolean empty) {
                                super.updateItem(item, empty);
                                setText(empty ? null : getString());
                                setGraphic(null);
                                setWrapText(true);

                                if (item != null && !empty) {
                                    switch (item.toString().toUpperCase()) {
                                        case "ERROR": {
                                            getStyleClass().remove("log-warning");
                                            getStyleClass().add("log-error");
                                            break;
                                        }
                                        case "WARNING": //fall through
                                        case "WARN": {
                                            getStyleClass().remove("log-error");
                                            getStyleClass().add("log-warning");
                                            break;
                                        }
                                        default: {
                                            getStyleClass().removeAll("log-warning", "log-error");
                                            break;
                                        }
                                    }
                                } else
                                    getStyleClass().removeAll("log-warning", "log-error");
                            }

                            private String getString() {
                                return getItem() == null ? "" : getItem().toString();
                            }
                        };

                        cell.addEventFilter(MouseEvent.MOUSE_CLICKED, cellDoubleClickHandler);
                        return cell;
                    }
                };
        levelCol.setCellFactory(levelCellFactory);
        timeCol.setCellFactory(cellFactory);
        classCol.setCellFactory(cellFactory);
        messageCol.setCellFactory(cellFactory);
        throwableCol.setCellFactory(cellFactory);

        getColumns().addAll(levelCol, timeCol, classCol, messageCol, throwableCol);

        levelCol.setCellValueFactory(new PropertyValueFactory<LogLine, String>("level"));
        timeCol.setCellValueFactory(new PropertyValueFactory<LogLine, String>("dateTime"));
        classCol.setCellValueFactory(new PropertyValueFactory<LogLine, String>("className"));
        messageCol.setCellValueFactory(new PropertyValueFactory<LogLine, String>("message"));
        throwableCol.setCellValueFactory(new PropertyValueFactory<LogLine, String>("exception"));

        levelCol.setPrefWidth(50);
        timeCol.setPrefWidth(100);
        classCol.setPrefWidth(50);
        messageCol.setPrefWidth(400);
        throwableCol.setPrefWidth(500);
//        messageCol.setPrefWidth(this.widthProperty().get() * 0.5);
//        throwableCol.setPrefWidth(this.widthProperty().get() * 0.4);
    }

}
