package be.iminds.ilabt.jfed.experimenter_gui.editor.properties.cmi;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.editor.bo.NodeDescription;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Singleton;

/**
 * User: twalcari
 * Date: 8/29/2014
 * Time: 09:28
 */
@Singleton
public class CMIManagerFactory {
    private static final Logger LOG = LoggerFactory.getLogger(CMIManagerFactory.class);
    private final GlobalCMIManager globalCmiManager;

    private final AuthorityList authorityList;
    private final JFedConfiguration config;
    private final ComponentManagerInfoFactory componentManagerInfoFactory;


    @Inject
    public CMIManagerFactory(AuthorityList authorityList, JFedConfiguration config, ComponentManagerInfoFactory componentManagerInfoFactory) {
        this.authorityList = authorityList;
        this.config = config;
        this.componentManagerInfoFactory = componentManagerInfoFactory;

        this.globalCmiManager = new GlobalCMIManager(authorityList, config, this.componentManagerInfoFactory);
    }

    public CMIManager getCMIManager(FXRspecNode node) {
        if (node.getNodeDescriptionId() == null)
            return globalCmiManager;
        else {
            NodeDescription nodeDescription = config.getNodeDescription(node.getNodeDescriptionId());
            if (nodeDescription != null) {
                return new FilteredCMIManager(nodeDescription, globalCmiManager);
            } else {
                LOG.warn("Could not find nodeDescription for nodeDescriptionId {}", node.getNodeDescriptionId());
                return globalCmiManager;
            }
        }
    }
}
