package be.iminds.ilabt.jfed.experimenter_gui.editor.properties.map;

import be.iminds.ilabt.jfed.experimenter_gui.editor.properties.cmi.CMIManager;
import be.iminds.ilabt.jfed.experimenter_gui.editor.properties.cmi.ComponentManagerInfo;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.FXUtils;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.cathive.fx.guice.GuiceFXMLLoader;
import javafx.collections.ObservableList;
import javafx.scene.Scene;
import javafx.stage.Screen;
import javafx.stage.Stage;
import javafx.stage.Window;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

/**
 * User: twalcari
 * Date: 1/8/2015
 * Time: 14:40
 */
@Singleton
public class MapDialogFactory {
    private static final Logger LOG = LoggerFactory.getLogger(MapDialogFactory.class);
    private static final String TESTBEDS_MAP_DIALOG_FXML = "TestbedsMapDialog.fxml";
    private static final String NODES_MAP_DIALOG_FXML = "NodesMapDialog.fxml";
    private static final String IMAGE_MAP_DIALOG_FXML = "ImageMapDialog.fxml";

    private final GuiceFXMLLoader loader;

    @Inject
    public MapDialogFactory(GuiceFXMLLoader loader) {
        this.loader = loader;
    }

    public ComponentManagerInfo showTestbedMapDialog(Window parent, CMIManager cmiManager) {
        try {
            GuiceFXMLLoader.Result result = loader.load(getClass().getResource(TESTBEDS_MAP_DIALOG_FXML));

            TestbedsMapDialog controller = result.getController();

            controller.setCmiManager(cmiManager);

            Scene scene = new Scene(result.getRoot());

            Stage dialogStage = new Stage();
            dialogStage.setScene(scene);

            dialogStage.setTitle("Map of available testbeds");
            dialogStage.setResizable(true);
            FXUtils.centerToOwner(parent, dialogStage);

            dialogStage.showAndWait();

            return controller.getResult();
        } catch (IOException e) {
            LOG.error("Error while creating testbeds map view", e);
            throw new RuntimeException(e);
        }
    }

    public FXRspecNode showNodeMapDialog(Window parent, List<FXRspecNode> nodes) {
        try {
            GuiceFXMLLoader.Result result = loader.load(getClass().getResource(NODES_MAP_DIALOG_FXML));

            NodesMapDialog controller = result.getController();

            controller.setNodes(nodes);

            Scene scene = new Scene(result.getRoot());

            Stage dialogStage = new Stage();
            dialogStage.setScene(scene);

            dialogStage.setTitle("Map of available nodes");
            dialogStage.setResizable(true);
            FXUtils.centerToOwner(parent, dialogStage);

            dialogStage.showAndWait();

            return controller.getResult();
        } catch (IOException e) {
            LOG.error("Error while creating testbeds map view", e);
            throw new RuntimeException(e);
        }
    }

    public FXRspecNode showNodeImageMapDialog(Window parent, ImageMap<GeniUrn> imageMap, Collection<FXRspecNode> availableNodes) {
        try {
            GuiceFXMLLoader.Result result = loader.load(getClass().getResource(IMAGE_MAP_DIALOG_FXML));

            ImageMapDialog controller = result.getController();

            controller.setNodesMap(imageMap, availableNodes);

            Scene scene = new Scene(result.getRoot());


            Stage dialogStage = new Stage();
            dialogStage.setScene(scene);
            dialogStage.setTitle("Image map of nodes");
            dialogStage.setResizable(true);

            dialogStage.sizeToScene();

            ObservableList<Screen> screens =
                    Screen.getScreensForRectangle(dialogStage.getX(), dialogStage.getY(), dialogStage.getWidth(), dialogStage.getHeight());

            double maxHeight =
                    screens.stream().mapToDouble(screen -> screen.getVisualBounds().getHeight()).min().orElse(dialogStage.getHeight());
            double maxWidth =
                    screens.stream().mapToDouble(screen -> screen.getVisualBounds().getWidth()).min().orElse(dialogStage.getWidth());

            dialogStage.setHeight(Double.min(dialogStage.getHeight(), maxHeight));
            dialogStage.setWidth(Double.min(dialogStage.getWidth(), maxWidth));

            // controller.setupStageSize(dialogStage);

            dialogStage.showAndWait();

            return controller.getResult();
        } catch (IOException e) {
            LOG.error("Error while creating testbeds map view", e);
            throw new RuntimeException(e);
        }
    }


}
