package be.iminds.ilabt.jfed.experimenter_gui.editor.ribbon_tabs;

import be.iminds.ilabt.jfed.experimenter_gui.controller.view.UpdateEvent;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonButton;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonComponentGroup;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;

import java.io.IOException;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 11:10
 */
public class ControlEditorRibbonTab extends EditorRibbonTab {

    private static final String CONTROL_EDITOR_RIBBON_TAB_FXML = "ControlEditorRibbonTab.fxml";

    private final RspecEditorRibbonTab rspecEditorRibbonTab;

    @FXML
    private RibbonButton zoomInButton;
    @FXML
    private RibbonButton zoomOutButton;
    @FXML
    private RibbonButton resetZoomButton;

    @FXML
    private RibbonComponentGroup definitionGroup;
    @FXML
    private RibbonButton addCommandButton;
    @FXML
    private RibbonButton addBarrierButton;

    public ControlEditorRibbonTab(RspecEditorRibbonTab rspecEditorRibbonTab) {
        this.rspecEditorRibbonTab = rspecEditorRibbonTab;

        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(CONTROL_EDITOR_RIBBON_TAB_FXML));

        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);
        fxmlLoader.setControllerFactory(aClass -> {
            if (aClass == GenericEditorRibbonComponentGroupController.class) {
                return new GenericEditorRibbonComponentGroupController(ControlEditorRibbonTab.this);
            }
            throw new RuntimeException("Unexpected class in ControllerFactory: " + aClass.getName());
        });


        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }

        setOnSelectionChanged(event -> {
            if (isSelected() && !getActiveExperimentEditor().isExperimentControllerVisible()) {
                FXModelRspec fxModelRspec = checkAndWarnForRawEditorDataLoss();
                if (fxModelRspec != null) {
                    getActiveExperimentEditor().switchToExperimentController(fxModelRspec);

                    getActiveExperimentEditor().getExperimentController().update(UpdateEvent.UPDATE_ZOOM, null);

                } else {
                    //revert to raw editor
                    getTabPane().getSelectionModel().select(ControlEditorRibbonTab.this.rspecEditorRibbonTab);
                }
            }
        });


        //The CSS on this rotate seems to be ignored.. adding it here works
        addBarrierButton.getGraphic().setRotate(90);
    }

    /*
            Zoom Actions
         */
    @FXML
    public void onZoomInAction() {
        getActiveExperimentEditor().getExperimentController().getController().zoomIn();
    }

    @FXML
    public void onZoomOutAction() {
        getActiveExperimentEditor().getExperimentController().getController().zoomOut();
    }

    @FXML
    public void onResetZoomAction() {
        getActiveExperimentEditor().getExperimentController().getController().resetZoom();
    }

    /*
        Control Actions
     */

    @FXML
    private void onAddCommandAction() {
        getActiveExperimentEditor().addCommand();
    }

    @FXML
    private void onAddBarrierAction() {
        getActiveExperimentEditor().addBarrier();
    }
}
