package be.iminds.ilabt.jfed.experimenter_gui.editor.ribbon_tabs;

import be.iminds.ilabt.jfed.experimenter_gui.editor.ExperimentEditor;
import be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation.RspecCheckReason;
import be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation.RspecValidationEngine;
import be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation.RspecValidationResult;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonTab;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 11:17
 */
public abstract class EditorRibbonTab extends RibbonTab {
    private final static Logger LOG = LoggerFactory.getLogger(EditorRibbonTab.class);

    protected final ObjectProperty<ExperimentEditor> activeExperimentEditor = new SimpleObjectProperty<>();
    protected final RspecValidationEngine validationEngine = new RspecValidationEngine();

    public ExperimentEditor getActiveExperimentEditor() {
        return activeExperimentEditor.get();
    }

    public void setActiveExperimentEditor(ExperimentEditor activeExperimentEditor) {
        this.activeExperimentEditor.set(activeExperimentEditor);
    }


    /**
     * Checks if there is data loss or other Rspec problem by switching from view.
     * If data loss would occur: warn the user and give him the choice if he wants to proceed..
     *
     * @return return an FXModelRspec if there is no data loss, or the user has accepted the warning. null if there should be reverted to the RawRspecEditor.
     */
    protected FXModelRspec checkAndWarnForRawEditorDataLoss() {
        if (getActiveExperimentEditor().isRawRspecEditorVisible()) {

            RequestRspecSource requestRspecSource = getActiveExperimentEditor().getRequestRspecSource();

            RspecValidationResult validationResult =
                    validationEngine.validateRSpec(RspecCheckReason.SWITCH_TO_VISUAL_EDITOR, requestRspecSource, getTabPane().getScene().getWindow());
            if(validationResult != null) {
                if (validationResult.isAbort())
                    return null;

                return (FXModelRspec) validationResult.getReplacementRequestRspecSource().getModelRspec(ModelRspecType.FX);
            }else{
                return (FXModelRspec) requestRspecSource.getModelRspec();
            }
        }

        if (getActiveExperimentEditor() != null && getActiveExperimentEditor().getRequestRspecSource() != null)
            return (FXModelRspec) getActiveExperimentEditor().getRequestRspecSource().getModelRspec(ModelRspecType.FX);
        else
            return null;
    }


}
