package be.iminds.ilabt.jfed.experimenter_gui.editor.ribbon_tabs;

import be.iminds.ilabt.jfed.experimenter_gui.canvas.EditableExperimentCanvas;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonButton;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;

import java.io.IOException;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 11:10
 */
public class TopologyEditorRibbonTab extends EditorRibbonTab {

    public static final String TOPOLOGY_EDITOR_RIBBON_TAB_FXML = "TopologyEditorRibbonTab.fxml";

    private final RspecEditorRibbonTab rspecEditorRibbonTab;

    @FXML
    private RibbonButton pasteButton;
    @FXML
    private RibbonButton copyButton;
    @FXML
    private RibbonButton duplicateButton;
    @FXML
    private RibbonButton zoomInButton;
    @FXML
    private RibbonButton zoomOutButton;
    @FXML
    private RibbonButton resetZoomButton;


    public TopologyEditorRibbonTab(RspecEditorRibbonTab rspecEditorRibbonTab) {
        this.rspecEditorRibbonTab = rspecEditorRibbonTab;

        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(TOPOLOGY_EDITOR_RIBBON_TAB_FXML));

        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);
        fxmlLoader.setControllerFactory(aClass -> {
            if (aClass == GenericEditorRibbonComponentGroupController.class) {
                return new GenericEditorRibbonComponentGroupController(TopologyEditorRibbonTab.this);
            }
            throw new RuntimeException("Unexpected class in ControllerFactory: " + aClass.getName());
        });


        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }

        setOnSelectionChanged(event -> {
            if (isSelected() && !getActiveExperimentEditor().isCanvasVisible()) {
                FXModelRspec fxModelRspec = checkAndWarnForRawEditorDataLoss();
                if (fxModelRspec != null) {
                    getActiveExperimentEditor().switchToEditableExperimentCanvas(fxModelRspec);
                    bindButtons();
                } else {
                    //revert to raw editor
                    getTabPane().getSelectionModel().select(TopologyEditorRibbonTab.this.rspecEditorRibbonTab);

                }
            }
        });

        activeExperimentEditor.addListener((observableValue, oldValue, newValue) -> {
            if (oldValue != null) {
                unbindButtons();
            }

            if (newValue != null) {
                if (newValue.getCanvas() != null) { //only bind if canvas is active
                    bindButtons();
                }
            }

        });

    }

    private void bindButtons() {
        assert (getActiveExperimentEditor() != null);
        assert (getActiveExperimentEditor().getCanvas() != null);
        EditableExperimentCanvas canvas = getActiveExperimentEditor().getCanvas();

        copyButton.disableProperty().bind(
                canvas.getSelectionProvider().selectedCanvasNodeProperty().isNull());
        duplicateButton.disableProperty().bind(
                canvas.getSelectionProvider().selectedCanvasNodeProperty().isNull());
        pasteButton.disableProperty().bind(canvas.getClipboard().emptyProperty());
    }

    private void unbindButtons() {
        copyButton.disableProperty().unbind();
        duplicateButton.disableProperty().unbind();
        pasteButton.disableProperty().unbind();
    }


    /*
        Edit section
     */

    @FXML
    private void onCopyAction() {
        getActiveExperimentEditor().getCanvas().copySelected();
    }

    @FXML
    private void onPasteAction() {
        getActiveExperimentEditor().getCanvas().paste();
    }

    @FXML
    private void onDuplicateAction() {
        getActiveExperimentEditor().getCanvas().duplicateSelected();
    }

    @FXML
    private void onAutoLayoutAction() {
        getActiveExperimentEditor().getModelRspecEditor().arrangeNodesOnCircle();
    }

    /*
        Zoom section
     */

    @FXML
    public void onZoomInAction() {
        getActiveExperimentEditor().getCanvas().zoomIn();
    }

    @FXML
    public void onZoomOutAction() {
        getActiveExperimentEditor().getCanvas().zoomOut();
    }

    @FXML
    public void onResetZoomAction() {
        getActiveExperimentEditor().getCanvas().setZoom(1);
    }
}
