package be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation;

import be.iminds.ilabt.jfed.rspec.convert.RspecConverter;
import be.iminds.ilabt.jfed.rspec.model.ModelRspec;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.model.RspecFactory;
import be.iminds.ilabt.jfed.rspec.model.RspecFactoryFactory;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import javafx.stage.Window;
import org.controlsfx.control.ButtonBar;
import org.controlsfx.control.action.Action;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.dialog.Dialogs;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 1/22/2015
 * Time: 17:01
 */
public class HasExpiresValidator implements RspecValidator {
    private static final Logger LOG = LoggerFactory.getLogger(HasExpiresValidator.class);

    @Override
    public RspecValidationResult validateRSpec(RspecCheckReason reason, RequestRspecSource requestRspecSource, Window dialogOwner) {
        if (requestRspecSource.getModelRspec().getExpiresString() != null) {
            String abortText = "Abort";
            String fixText = "Remove \"expires\" and continue";
            String ignoreText = "Keep as is and continue";

            boolean showIgnore = false;
            switch (reason) {
                case SWITCH_TO_VISUAL_EDITOR: {
                    return null; //no action needed
                }
                case RUN_EXPERIMENT: {
                    abortText = "Abort and continue editing";
                    ignoreText = "Ignore and Run Experiment Anyway";
                    fixText = "Remove \"expires\" and Run Experiment";
                    showIgnore = true;
                    break;
                }
                case CHECK_RSPEC: {
                    abortText = "Keep as is";
                    fixText = "Remove \"expires\"";
                    showIgnore = false;
                    break;
                }
                case OPENED_FILE: {
                    abortText = "Keep as is";
                    fixText = "Remove \"expires\"";
                    showIgnore = false;
                    break;
                }
            }

            Action fix = new DialogAction(fixText, ButtonBar.ButtonType.YES, false, true, true);
            Action ignore = new DialogAction(ignoreText, ButtonBar.ButtonType.CANCEL_CLOSE, true, true, false);
            Action abort = new DialogAction(abortText, ButtonBar.ButtonType.CANCEL_CLOSE, false, true, false);

            Dialogs d = Dialogs.create().owner(dialogOwner)
                    .message("Problem in RSpec:\n" +
                                    "This RSpec has the \"expires\" attribute. This attribute determines the expiration time of the slivers when sent to certain AM's.\n" +
                                    "jFed add and removes this attribute when needed, but when using this RSpec with older jFed versions, or other tools, it can cause problems.\n" +
                                    "It is advised to remove the \"expires\" attribute.\n\n" +
                                    "Do you want to remove the \"expires\" attribute?"
                    )
                    .title("RSpec has \"expires\" attribute");
            if (showIgnore)
                d = d.actions(fix, ignore, abort);
            else
                d = d.actions(fix, abort);
            Action chosenAction = d.showError();

            if (chosenAction == fix) {
                LOG.info("Removing expires attribute");

                FXModelRspec newModel = (FXModelRspec) requestRspecSource.getModelRspec().copy();
                newModel.setExpires((String)null);

                return new RspecValidationResult(newModel);
            } else if (chosenAction == ignore) {
                return new RspecValidationResult(requestRspecSource, false /* == ignore */);

            } else if (chosenAction == abort) {
                return new RspecValidationResult(requestRspecSource, true/* == abort*/);
            } else {
                throw new RuntimeException("Unknown chosen action: " + chosenAction);
            }
        }

        return null;
    }
}
