package be.iminds.ilabt.jfed.experimenter_gui.slice;

import be.iminds.ilabt.jfed.experimenter_gui.call_gui.TasksWindow;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.ControllerNodeFactory;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.ControllerUI;
import be.iminds.ilabt.jfed.experimenter_gui.slice.errors.ErrorsView;
import be.iminds.ilabt.jfed.experimenter_gui.slice.raw.RawSliceView;
import be.iminds.ilabt.jfed.experimenter_gui.util.ImageUtil;
import be.iminds.ilabt.jfed.experimenter_gui.util.SSHTerminalUtil;
import be.iminds.ilabt.jfed.log.cache.ApiCallDetailsCache;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.rspec.rspec_source.ManifestRspecSource;
import be.iminds.ilabt.jfed.util.ProxyPreferencesManager;

import javax.inject.Inject;
import javax.inject.Singleton;

/**
 * User: twalcari
 * Date: 1/30/2015
 * Time: 11:50
 */
@Singleton
public class ExperimentViewsFactory {

    private final TasksWindow tasksWindow;
    private final ApiCallDetailsCache apiCallDetailsCache;

    private final SSHTerminalUtil sshTerminalUtil;
    private final JFedGuiPreferences jFedPreferences;


    private final AuthorityListModel authorityListModel;
    private final SliceExperimentCanvasFactory sliceExperimentCanvasFactory;

    private final NodePropertiesDialogFactory nodePropertiesDialogFactory;
    private final ImageUtil imageUtil;
    private final GeniUserProvider geniUserProvider;
    private final ProxyPreferencesManager proxyPreferencesManager;
    private final ControllerNodeFactory controllerNodeFactory;


    @Inject
    public ExperimentViewsFactory(ApiCallDetailsCache apiCallDetailsCache, TasksWindow tasksWindow,
                                  SSHTerminalUtil sshTerminalUtil, JFedGuiPreferences jFedPreferences,
                                  AuthorityListModel authorityListModel, SliceExperimentCanvasFactory sliceExperimentCanvasFactory,
                                  NodePropertiesDialogFactory nodePropertiesDialogFactory, ImageUtil imageUtil,
                                  GeniUserProvider geniUserProvider, ProxyPreferencesManager proxyPreferencesManager,
                                  ControllerNodeFactory controllerNodeFactory) {
        this.apiCallDetailsCache = apiCallDetailsCache;
        this.tasksWindow = tasksWindow;
        this.sshTerminalUtil = sshTerminalUtil;
        this.jFedPreferences = jFedPreferences;
        this.authorityListModel = authorityListModel;
        this.sliceExperimentCanvasFactory = sliceExperimentCanvasFactory;
        this.nodePropertiesDialogFactory = nodePropertiesDialogFactory;
        this.imageUtil = imageUtil;
        this.geniUserProvider = geniUserProvider;
        this.proxyPreferencesManager = proxyPreferencesManager;
        this.controllerNodeFactory = controllerNodeFactory;
    }

    public ErrorsView createErrorsView() {
        return new ErrorsView(tasksWindow, apiCallDetailsCache);
    }

    public RawSliceView createRawSliceView(Experiment experiment) {
        return new RawSliceView(experiment, sshTerminalUtil, jFedPreferences);
    }

    public ModelSliceView createModelSliceView(ExperimentController experimentController, ManifestRspecSource canvasRspecSource) {
        return new ModelSliceView(experimentController, canvasRspecSource, authorityListModel, sliceExperimentCanvasFactory);
    }

    public ControllerUI createControllerUi(Experiment experiment, ManifestRspecSource canvasRspecSource, DetailsTabHost detailsTabHost) {
        return new ControllerUI(experiment, canvasRspecSource, detailsTabHost, nodePropertiesDialogFactory, imageUtil,
                geniUserProvider, proxyPreferencesManager, controllerNodeFactory, jFedPreferences);
    }
}
