package be.iminds.ilabt.jfed.experimenter_gui.slice.actions;

import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.tasks.ExperimentTasksFactory;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityInfo;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.application.Platform;
import javafx.scene.control.TextInputDialog;
import javafx.stage.Window;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Optional;
import java.util.regex.Pattern;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 15:05
 */
public class CreateDiskImageAction extends ExperimentAction{
    private static final Logger LOG = LoggerFactory.getLogger(CreateDiskImageAction.class);

    private static final Pattern DISK_IMAGE_PATTERN = Pattern.compile("[A-Za-z0-9_-]{1,30}");

    private final FXRspecNode node;
    private final AuthorityList authorityList;


    CreateDiskImageAction(Window parentWindow, ExperimentController experimentController, FXRspecNode node,
                                 AuthorityList authorityList, ExperimentTasksFactory experimentTasksFactory) {
        super(parentWindow, experimentController, experimentTasksFactory);
        this.node = node;
        this.authorityList = authorityList;
    }

    @Override
    public void run() {
        assert (Platform.isFxApplicationThread());

        String imageName = null;

        while (imageName == null) {
            TextInputDialog textInputDialog = new TextInputDialog();
            textInputDialog.initOwner(parentWindow);
            textInputDialog.setTitle("Create a Disk Image");
            textInputDialog.setHeaderText("Create a Disk Image");
            textInputDialog.setContentText("Please provide a name for the node image:\n" +
                    "(Allowed characters: [A-Za-z0-9_-], Maximum length: 30)");

            Optional<String> result = textInputDialog.showAndWait();

            //if the input is still null, the user has cancelled the input-dialog. Exit!
            if (!result.isPresent())
                return;

            imageName = result.get();

            if (!DISK_IMAGE_PATTERN.matcher(imageName).matches()) {
                JFDialogs.create()
                        .owner(parentWindow)
                        .message("The image name may only contain the following characters: [A-Za-z0-9_-], and must be less than 30 characters long.")
                        .masthead("The provided image name is invalid.")
                        .showError();
                imageName = null;
            }
        }

        SfaAuthority auth = authorityList.finder().findByUrn(node.getComponentManagerId(), AuthorityFinder.Purpose.CREATE_SLIVER); //same as createsliver
        if (auth == null) {
            LOG.error("Could not fetch authority info for node {} with componentManagerId {}",
                    node.getClientId(), node.getComponentManagerId());
            return;
        }

        experimentController.submitExperimentTask(experimentTasksFactory.createCreateDiskImageTask(experimentController, auth, node, imageName));

    }
}
