package be.iminds.ilabt.jfed.experimenter_gui.slice.raw;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentChangeListener;
import be.iminds.ilabt.jfed.experimenter_gui.slice.SliceState;
import be.iminds.ilabt.jfed.experimenter_gui.ui.code.XmlCodeView;
import be.iminds.ilabt.jfed.experimenter_gui.util.DateTimeUtils;
import be.iminds.ilabt.jfed.experimenter_gui.util.FileUtils;
import be.iminds.ilabt.jfed.experimenter_gui.util.SSHTerminalUtil;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.StatusDetails;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.rspec.model.BasicStringRspec;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.rspec.rspec_source.ManifestRspecSource;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.XmlUtil;
import javafx.application.Platform;
import javafx.beans.InvalidationListener;
import javafx.beans.WeakInvalidationListener;
import javafx.beans.binding.Bindings;
import javafx.beans.binding.DoubleBinding;
import javafx.beans.property.ReadOnlyIntegerWrapper;
import javafx.beans.property.ReadOnlyStringWrapper;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.control.cell.TextFieldTableCell;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.FileChooser;
import javafx.util.Callback;
import javafx.util.StringConverter;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * User: twalcari
 * Date: 2/6/14
 * Time: 8:21 AM
 */
public class RawSliceView extends StackPane implements ExperimentChangeListener {
    private static final Logger LOG = LoggerFactory.getLogger(RawSliceView.class);

    private static final String RAW_SLICE_VIEW_FXML = "RawSliceView.fxml";
    private static final String COMBINED_MANIFEST = "Combined Manifest RSpec";

    private final Experiment experiment;
    private final SSHTerminalUtil sshTerminalUtil;
    private final JFedGuiPreferences jFedPreferences;
    private final StringConverter<Object> manifestComboboxStringConverter = new StringConverter<Object>() {
        @Override
        public String toString(Object object) {
            if (object instanceof SfaAuthority)
                return ((SfaAuthority) object).getHrn();
            else if (object instanceof String)
                return (String) object;
            else
                throw new UnsupportedOperationException("Only String and SfaAuthority are supported");
        }

        @Override
        public Object fromString(String string) {
            throw new UnsupportedOperationException();
        }
    };

    @FXML
    private HBox initializingBox;
    @FXML
    private ScrollPane testbedsScrollPane;
    @FXML
    private VBox testbedsVBox;

    @FXML
    private TableView<Sliver> sliversTableView;

    @FXML
    private TableColumn<Sliver, String> sliverTestbedTableColumn;

    @FXML
    private TableColumn<Sliver, GeniUrn> sliverIdTableColumn;

    @FXML
    private TableColumn<Sliver, Instant> sliverExpirationTimeTableColumn;

    @FXML
    private TableColumn<Sliver, StatusDetails> sliverStatusTableColumn;

    @FXML
    private TableView<RspecNodeWithLoginInfo> nodesTableView;

    @FXML
    private TableColumn<RspecNodeWithLoginInfo, String> nodeNameTableColumn;

    @FXML
    private TableColumn<RspecNodeWithLoginInfo, String> hostnameTableColumn;

    @FXML
    private TableColumn<RspecNodeWithLoginInfo, Number> portTableColumn;

    @FXML
    private TableColumn<RspecNodeWithLoginInfo, String> usernameTableColumn;

    @FXML
    private TableColumn<RspecNodeWithLoginInfo, Boolean> loginTableColumn;

    /**
     * this combobox should only contain String of SfaAuthority-objects!
     */
    @FXML
    private ComboBox<Object> manifestCombobox;
    @FXML
    private Label manifestLabel;

    @FXML
    private XmlCodeView manifestXmlView;


    private Slice slice = null;

    private File prevSaveFile = null;


    public RawSliceView(Experiment experiment, SSHTerminalUtil sshTerminalUtil, JFedGuiPreferences jFedPreferences) {
        this.experiment = experiment;
        this.sshTerminalUtil = sshTerminalUtil;
        this.jFedPreferences = jFedPreferences;

        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(RAW_SLICE_VIEW_FXML));

        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);

        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }

        testbedsVBox.prefWidthProperty().bind(testbedsScrollPane.widthProperty().subtract(15));

        //initialize slivers tableview
        sliverTestbedTableColumn.setCellValueFactory(t -> new ReadOnlyStringWrapper(t.getValue().getAuthority().getName()));

        sliverIdTableColumn.setCellValueFactory(new PropertyValueFactory<>("urn"));


        sliverExpirationTimeTableColumn.setCellValueFactory(new PropertyValueFactory<>("expirationDate"));
        sliverExpirationTimeTableColumn.setCellFactory(TextFieldTableCell.<Sliver, Instant>forTableColumn(new StringConverter<Instant>() {
            @Override
            public String toString(Instant date) {
                if (date != null) {
                    return DateTimeUtils.DATE_HOUR_MINUTES_SECONDS_FORMATTER.format(LocalDateTime.ofInstant(date, ZoneId.systemDefault()));
                } else {
                    return null;
                }
            }

            @Override
            public Instant fromString(String s) {
                return ZonedDateTime.parse(s, DateTimeUtils.DATE_HOUR_MINUTES_SECONDS_FORMATTER).toInstant();
            }
        }));


        sliverStatusTableColumn.setCellValueFactory(new PropertyValueFactory<>("status"));


        sliverStatusTableColumn.setCellFactory(sliverSliverStatusTableColumn -> new StatusTableCell());

        //set table widths
        DoubleBinding sliverTableWidthLeft =
                sliversTableView.widthProperty().subtract(2).subtract(sliverStatusTableColumn.widthProperty());


        sliverTestbedTableColumn.prefWidthProperty().bind(sliverTableWidthLeft.multiply(0.25));
        sliverIdTableColumn.prefWidthProperty().bind(sliverTableWidthLeft.multiply(0.5));
        sliverExpirationTimeTableColumn.prefWidthProperty().bind(sliverTableWidthLeft.multiply(0.25));

        //initialize nodes tableview

        nodeNameTableColumn.setCellValueFactory(t -> new ReadOnlyStringWrapper(t.getValue().getRspecNode().getClientId()));

        hostnameTableColumn.setCellValueFactory(t -> {
            if (t.getValue().getLoginService() != null) {
                return new ReadOnlyStringWrapper(t.getValue().getLoginService().getHostname());
            } else
                return null;
        });

        portTableColumn.setCellValueFactory(new Callback<TableColumn.CellDataFeatures<RspecNodeWithLoginInfo, Number>, ObservableValue<Number>>() {
            @Override
            public ObservableValue<Number> call(TableColumn.CellDataFeatures<RspecNodeWithLoginInfo, Number> t) {
                if (t.getValue().getLoginService() != null) {
                    try {
                        return new ReadOnlyIntegerWrapper(t.getValue().getLoginService().getPort());
                    } catch (NumberFormatException ex) {
                        LOG.error("Could not convert loginService port to number.", ex);
                        return null;
                    }
                } else
                    return null;
            }
        });

        usernameTableColumn.setCellValueFactory(t -> {
            if (t.getValue().getLoginService() != null) {
                return new ReadOnlyStringWrapper(t.getValue().getLoginService().getUsername());
            } else
                return null;
        });

        loginTableColumn.setCellValueFactory(
                p -> new SimpleBooleanProperty(p.getValue().getLoginService() != null)
        );

        loginTableColumn.setCellFactory(t -> new TableCell<RspecNodeWithLoginInfo, Boolean>() {
            @Override
            protected void updateItem(Boolean item, boolean empty) {
                if (empty) {
                    setText(null);
                    setGraphic(null);
                } else {
                    final Button loginButton = new Button("Login");
                    loginButton.setDisable(!item);
                    loginButton.setOnAction(actionEvent -> sshTerminalUtil.launchSSHTerminal(((RspecNodeWithLoginInfo) getTableRow().getItem()).getLoginService()));
                    loginButton.setGraphic(GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TERMINAL, Color.BLACK));
                    setGraphic(loginButton);
                    setContentDisplay(ContentDisplay.GRAPHIC_ONLY);
                }
            }
        });

        nodesTableView.prefHeightProperty().bind(Bindings.max(100,
                Bindings.size(nodesTableView.getItems()).multiply(28).add(25 /*header */ + 2 /*safety margin */)));

        //set tablecolumn widths
        nodeNameTableColumn.prefWidthProperty().bind(
                nodesTableView.widthProperty().subtract(loginTableColumn.getPrefWidth() + 2).multiply(0.3));
        hostnameTableColumn.prefWidthProperty().bind(
                nodesTableView.widthProperty().subtract(loginTableColumn.getPrefWidth() + 2).multiply(0.3));
        portTableColumn.prefWidthProperty().bind(
                nodesTableView.widthProperty().subtract(loginTableColumn.getPrefWidth() + 2).multiply(0.2));
        usernameTableColumn.prefWidthProperty().bind(
                nodesTableView.widthProperty().subtract(loginTableColumn.getPrefWidth() + 2).multiply(0.2));

        visibleProperty().addListener(observable -> {
            if (isVisible() && slice != null) {
                sliversTableView.itemsProperty().unbind();
                //sliversTableView.getItems().clear();
                sliversTableView.itemsProperty().bind(slice.sliversProperty());

            }
        });

        manifestCombobox.setConverter(manifestComboboxStringConverter);
        manifestCombobox.getItems().add(COMBINED_MANIFEST);
        manifestCombobox.getSelectionModel().selectFirst();
        manifestCombobox.getSelectionModel().selectedItemProperty().addListener(observable -> updateManifestView());

        manifestXmlView.prefWidthProperty().bind(testbedsVBox.widthProperty().subtract(42));


        experiment.addExperimentChangeListener(this);
        if (experiment.getSlice() != null) {
            this.initializeSlice(experiment.getSlice());
        }


    }

    private final InvalidationListener manifestInvalidationListener = observable -> {
        if (Platform.isFxApplicationThread()) {
            updateView();
        } else {
            Platform.runLater(this::updateView);
        }
    };

    private void updateView() {
        //if we don't have a manifest, we're still initializing
        if (slice.getManifestRspec() == null)
            return;

        initializingBox.setVisible(false);
        initializingBox.setManaged(false);
        testbedsScrollPane.setVisible(true);

        if (slice.getManifestRspec().getModelRspec() != null) {
            ObservableList<RspecNodeWithLoginInfo> rspecNodeWithLoginInfos = FXCollections.observableArrayList();

            for (FXRspecNode node : ((FXModelRspec) slice.getManifestRspec().getModelRspec(ModelRspecType.FX)).getNodes()) {
                List<BasicStringRspec.LoginService> loginServices = slice.getManifestRspec().getNodeLoginInfo(node);
                if (loginServices == null || loginServices.isEmpty()) {
                    rspecNodeWithLoginInfos.add(new RspecNodeWithLoginInfo(node, null));
                } else {
                    for (BasicStringRspec.LoginService loginService : loginServices) {
                        rspecNodeWithLoginInfos.add(new RspecNodeWithLoginInfo(node, loginService));
                    }
                }
            }

            FXCollections.sort(rspecNodeWithLoginInfos);

            nodesTableView.getItems().setAll(rspecNodeWithLoginInfos);
        } else {
            LOG.warn("Could not retrieve FXModelRspec from slice.getManifestRspec()!");
        }

        //update the available manifest rspecs:

        Object oldSelection = manifestCombobox.getSelectionModel().getSelectedItem();

        List<SfaAuthority> manifestAMsList = new ArrayList<>(slice.getManifestComponentManagers());
        manifestAMsList.sort(Comparator.comparing(SfaAuthority::getName));

        ObservableList<Object> allManifests = FXCollections.observableArrayList();
        allManifests.add(COMBINED_MANIFEST);
        allManifests.addAll(manifestAMsList);

        manifestCombobox.setItems(allManifests);
        manifestCombobox.getSelectionModel().select(oldSelection);

        updateManifestView();
    }

    @FXML
    private void updateManifestView() {
        //if we don't have a manifest, we're still initializing
        if (slice == null || slice.getManifestRspec() == null)
            return;

        assert Platform.isFxApplicationThread();

        ManifestRspecSource manifestRspecSource = slice.getManifestRspec();

        if (manifestCombobox.getSelectionModel().getSelectedItem() != COMBINED_MANIFEST) {
            assert manifestCombobox.getSelectionModel().getSelectedItem() instanceof SfaAuthority;
            manifestRspecSource = slice.getSpecificManifestRspec(
                    ((SfaAuthority) manifestCombobox.getSelectionModel().getSelectedItem()));
        }

        if (manifestRspecSource != null) {

            manifestXmlView.setContent(XmlUtil.formatXmlFromString(manifestRspecSource.getRspecXmlString()));
        } else {
            manifestXmlView.setContent("No manifest available");
        }
    }

    @FXML
    private void saveManifest() {
        //if we don't have a manifest, we're still initializing
        if (slice == null || slice.getManifestRspec() == null)
            return;

        assert Platform.isFxApplicationThread();
        ManifestRspecSource manifestRspecSource = slice.getManifestRspec();

        if (manifestCombobox.getSelectionModel().getSelectedItem() != COMBINED_MANIFEST) {
            assert manifestCombobox.getSelectionModel().getSelectedItem() instanceof SfaAuthority;
            manifestRspecSource = slice.getSpecificManifestRspec(
                    ((SfaAuthority) manifestCombobox.getSelectionModel().getSelectedItem()));
        }

        if (manifestRspecSource == null) {
            LOG.warn("No manifest available to save");
            return;
        }

        String selectedTitleString =
                manifestComboboxStringConverter.toString(manifestCombobox.getSelectionModel().getSelectedItem());
        String manifestXmlString = XmlUtil.formatXmlFromString(manifestRspecSource.getRspecXmlString());

        FileChooser fc = new FileChooser();
        fc.setTitle("Save Manifest RSpec");
        fc.getExtensionFilters().add(FileUtils.MANIFEST_RSPEC_EXTENSION_FILTER);
        fc.setInitialFileName(selectedTitleString + FileUtils.MANIFEST_RSPEC_EXTENSION);


        //set the default save dir
        if (System.getProperty("user.home") != null)
            fc.setInitialDirectory(new File(System.getProperty("user.home")));

        if (jFedPreferences.containsPreference(GuiPreferenceKey.PREF_RSPECFOLDER_MANIFEST)) {
            File dir = new File(jFedPreferences.getString(GuiPreferenceKey.PREF_RSPECFOLDER_MANIFEST));
            if (dir.exists()) {
                fc.setInitialDirectory(dir);
            }
        } else {
            if (jFedPreferences.containsPreference(GuiPreferenceKey.PREF_RSPECFOLDER_MANIFEST)) {
                File dir = new File(jFedPreferences.getString(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST));
                if (dir.exists()) {
                    fc.setInitialDirectory(dir);
                }
            }
        }


        if (prevSaveFile != null) {
            fc.setInitialDirectory(prevSaveFile.getParentFile());
//            fc.setInitialFileName(prevSaveFile.getName());
        }

        File saveFile = fc.showSaveDialog(this.getScene().getWindow());

        if (saveFile == null)
            return;

        //save folder in which the file was saved for future file selector dialogs
        jFedPreferences.setString(GuiPreferenceKey.PREF_RSPECFOLDER_MANIFEST,
                saveFile.getParentFile().getAbsolutePath());

        if (!saveFile.getAbsolutePath().endsWith(FileUtils.MANIFEST_RSPEC_EXTENSION))
            saveFile = new File(saveFile.getAbsolutePath() + FileUtils.MANIFEST_RSPEC_EXTENSION);

        final String fileContents = manifestXmlString;
        final File finalSaveFile = saveFile;
        final javafx.concurrent.Task<Void> saveTask = new javafx.concurrent.Task<Void>() {

            @Override
            protected Void call() throws Exception {
                updateMessage("Saving manifest in \"" + finalSaveFile.getAbsolutePath() + "\"");

                try (FileWriter writer = new FileWriter(finalSaveFile)) {
                    writer.write(fileContents);
                    writer.flush();
                    writer.close();
                } catch (final IOException ex) {
                    LOG.error("Error while saving experiment to file '" + finalSaveFile.getAbsolutePath() + "'", ex);
                    Platform.runLater(new Runnable() {
                        @Override
                        public void run() {
                            JFDialogs.create()
                                    .owner(this)
                                    .message("An error occurred while saving the rspec.")
                                    .masthead("Error")
                                    .title("Error")
                                    .showException(ex);

                        }
                    });
                }
                updateMessage("Successfully saved experiment in \"" + finalSaveFile.getAbsolutePath() + "\"");

                return null;
            }
        };
        //saveTask.messageProperty().addListener(observable -> setStatus(saveTask.getMessage()));

        new Thread(saveTask).start();
    }

    public Slice getSlice() {
        return slice;
    }

    private void initializeSlice(Slice slice) {
        // a slice can only be set one time, after  that, the field becomes immutable
        assert (this.slice == null);
        this.slice = slice;

        LOG.debug("Initializing RawSliceView with slice {}", slice.getUrn());

        Platform.runLater(() -> {
            sliversTableView.itemsProperty().bind(slice.sliversProperty());

            sliversTableView.prefHeightProperty().bind(Bindings.max(100,
                    Bindings.size(sliversTableView.getItems()).multiply(25).add(25 /*header*/ + 2 /* safety margin */)));

            slice.manifestRspecProperty().addListener(new WeakInvalidationListener(manifestInvalidationListener));
            updateView();
        });
    }

    @Override
    public void onSliceStateChange(SliceState newState) {
        if (this.slice == null && newState != SliceState.PENDING) {
            assert (experiment.getSlice() != null);
            initializeSlice(experiment.getSlice());
        }
    }

    private static class StatusTableCell extends TableCell<Sliver, StatusDetails> {
        private static String statusToStyleClass(StatusDetails status) {
            return statusToStyleClass(status.getGlobalStatus());
        }

        private static String statusToStyleClass(StatusDetails.SliverStatus status) {
            switch (status) {
                case READY:
                    return "testbed-status-ready";

                case CHANGING:
                    return "testbed-status-changing";

                case FAIL:
                    return "testbed-status-fail";

                case UNALLOCATED:
                    return "testbed-status-unallocated";

                case UNINITIALISED:
                    return "testbed-status-uninitialised";

                case UNKNOWN:
                    return "testbed-status-unknown";
                default:
                    throw new IllegalArgumentException();
            }
        }

        @Override
        protected void updateItem(StatusDetails item, boolean empty) {
            super.updateItem(item, empty);

            if (LOG.isTraceEnabled())
                LOG.trace("Update sliver status with {} and {}", item, empty);

            if (empty)
                return;

            setText(item.getGlobalStatus().name());
            for (StatusDetails.SliverStatus s : StatusDetails.SliverStatus.values()) {
                getStyleClass().remove(statusToStyleClass(s));
            }
            getStyleClass().add(statusToStyleClass(item));
        }
    }


    private final class RspecNodeWithLoginInfo implements Comparable<RspecNodeWithLoginInfo> {
        private final FXRspecNode rspecNode;
        private final BasicStringRspec.LoginService loginService;

        private RspecNodeWithLoginInfo(FXRspecNode rspecNode, BasicStringRspec.LoginService loginService) {
            this.rspecNode = rspecNode;
            this.loginService = loginService;
        }

        public FXRspecNode getRspecNode() {
            return rspecNode;
        }

        public BasicStringRspec.LoginService getLoginService() {
            return loginService;
        }

        @Override
        public int compareTo(RspecNodeWithLoginInfo o) {
            if (o == null)
                throw new NullPointerException();

            int result = getRspecNode().getUniqueId().compareTo(o.getRspecNode().getUniqueId());

            if (result == 0 && getLoginService() != null) {
                if (o.getLoginService() == null)
                    result = -1;
                else {
                    result = getLoginService().getUsername().compareTo(o.getLoginService().getUsername());

                    if (result == 0) {
                        result = getLoginService().getPort() - o.getLoginService().getPort();
                    }

                    if (result == 0) {
                        result = getLoginService().getHostname().compareTo(o.getLoginService().getHostname());
                    }

                    if (result == 0) {
                        result =
                                getLoginService().getAuthentication().compareTo(o.getLoginService().getAuthentication());
                    }
                }
            } else if (o.getLoginService() != null) {
                result = 1;
            }

            return result;
        }
    }
}
