package be.iminds.ilabt.jfed.experimenter_gui.slice.ribbon_tabs;

import be.iminds.ilabt.jfed.experimenter_gui.dialogs.DialogsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentBindings;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonButton;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import javafx.beans.binding.Bindings;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.fxml.FXML;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 17:20
 */
public class AdvancedSliceRibbonComponentGroupController {
    private final static Logger LOG = LoggerFactory.getLogger(AdvancedSliceRibbonComponentGroupController.class);

    private final SliceRibbonTab sliceRibbonTab;

    private final DialogsFactory dialogsFactory;

    @FXML
    private RibbonButton restartExperimentButton;
    @FXML
    private RibbonButton editSshKeysExperimentButton;
    @FXML
    private RibbonButton shareSliceButton;


    private final BooleanProperty isDemo =
            new SimpleBooleanProperty(new JFedGuiPreferences().getBoolean(GuiPreferenceKey.PREF_IS_DEMO,false));

    public AdvancedSliceRibbonComponentGroupController(SliceRibbonTab sliceRibbonTab, DialogsFactory dialogsFactory) {
        this.sliceRibbonTab = sliceRibbonTab;
        this.dialogsFactory = dialogsFactory;

        sliceRibbonTab.activeExperimentViewControllerProperty().addListener((observableValue, oldValue, newValue) -> {
            if (oldValue != null) {
                restartExperimentButton.disableProperty().unbind();
                editSshKeysExperimentButton.disableProperty().unbind();
                shareSliceButton.disableProperty().unbind();
            }

            if (newValue != null) {
                BooleanProperty manifestNotAvailable =
                        ExperimentBindings.createManifestNotAvailableBinding(newValue.getExperiment());

                restartExperimentButton.disableProperty().bind(Bindings.or(manifestNotAvailable, isDemo));
                editSshKeysExperimentButton.disableProperty().bind(Bindings.or(manifestNotAvailable, isDemo));
                shareSliceButton.disableProperty().bind(Bindings.or(manifestNotAvailable, isDemo));
            }

        });
    }


    @FXML
    private void onRebootExperimentAction() {
        dialogsFactory.showRebootDialog(restartExperimentButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController());
    }

    @FXML
    private void onEditSshKeysExperimentAction() {
        dialogsFactory.showEditSshKeysDialog(editSshKeysExperimentButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController());
    }

    @FXML
    private void onShareSliceAction() {
        dialogsFactory.showShareSliceDialog(shareSliceButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController());
    }
}
