package be.iminds.ilabt.jfed.experimenter_gui.slice.ribbon_tabs;

import be.iminds.ilabt.jfed.experimenter_gui.controller.view.dialogs.AddMultipleCommandDialog;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.dialogs.FastForwardDialog;
import be.iminds.ilabt.jfed.experimenter_gui.controller.view.dialogs.InstantCommandDialog;
import be.iminds.ilabt.jfed.experimenter_gui.dialogs.DialogsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.actions.ExperimentActionsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.tasks.ExperimentTasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonButton;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonComponentGroup;
import be.iminds.ilabt.jfed.rspec.model.controller.Timestamp;
import javafx.beans.binding.Bindings;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;

import java.io.IOException;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 17:31
 */
public class ControlViewerRibbonTab extends SliceRibbonTab {

    private static final String CONTROL_VIEWER_RIBBON_TAB_FXML = "ControlViewerRibbonTab.fxml";

    @FXML
    private RibbonComponentGroup controllerGroup;
    @FXML
    private RibbonButton startControllerButton;
    @FXML
    private RibbonButton pauseControllerButton;
    @FXML
    private RibbonButton stopControllerButton;
    //    @FXML
//    private RibbonButton rewindControllerButton;
    @FXML
    private RibbonButton fastForwardControllerButton;


    @FXML
    private RibbonButton zoomInButton;
    @FXML
    private RibbonButton zoomOutButton;
    @FXML
    private RibbonButton resetZoomButton;

    @FXML
    private RibbonComponentGroup definitionGroup;
    @FXML
    private RibbonButton addCommandButton;
    @FXML
    private RibbonButton addBarrierButton;


    public ControlViewerRibbonTab(DialogsFactory dialogsFactory, ExperimentActionsFactory experimentActionsFactory, ExperimentTasksFactory experimentTasksFactory) {
        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(CONTROL_VIEWER_RIBBON_TAB_FXML));

        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);
        fxmlLoader.setControllerFactory(aClass -> {
            if (aClass == GenericSliceRibbonComponentGroupController.class) {
                return new GenericSliceRibbonComponentGroupController(ControlViewerRibbonTab.this, experimentActionsFactory, experimentTasksFactory, dialogsFactory);
            } else if (aClass == AdvancedSliceRibbonComponentGroupController.class) {
                return new AdvancedSliceRibbonComponentGroupController(ControlViewerRibbonTab.this, dialogsFactory);
            }
            throw new RuntimeException("Unexpected class in ControllerFactory: " + aClass.getName());
        });


        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }

        setOnSelectionChanged(event -> {
            if (isSelected() && !getActiveExperimentViewController().isControllerVisible()) {
                    getActiveExperimentViewController().switchToExperimentController();
            }
        });


        activeExperimentViewControllerProperty().addListener((observableValue, oldValue, newValue) -> {

            if (oldValue != null) {
                controllerGroup.disableProperty().unbind();
                startControllerButton.disableProperty().unbind();
                pauseControllerButton.disableProperty().unbind();
                stopControllerButton.disableProperty().unbind();
            }

            if (newValue != null) {

                if (newValue.getExperimentController() != null) {
                    controllerGroup.disableProperty().bind(
                            Bindings.not(newValue.getControllerUi().onlineProperty()));

                    startControllerButton.disableProperty().bind(
                            newValue.getControllerUi().getModel().startedProperty());
                    pauseControllerButton.disableProperty().bind(
                            newValue.getControllerUi().getModel().stoppedProperty());
                    stopControllerButton.disableProperty().bind(
                            newValue.getControllerUi().getModel().elapsedTimeProperty().isEqualTo(0L));
                }
            }
        });


        //The CSS on this rotate seems to be ignored.. adding it here works
        addBarrierButton.getGraphic().setRotate(90);

    }

    /*
            Zoom Actions
         */
    @FXML
    public void onZoomInAction() {
        getActiveExperimentViewController().getControllerUi().getController().zoomIn();
    }

    @FXML
    public void onZoomOutAction() {
        getActiveExperimentViewController().getControllerUi().getController().zoomOut();
    }

    @FXML
    public void onResetZoomAction() {
        getActiveExperimentViewController().getControllerUi().getController().resetZoom();
    }

    /*
        Controller definition Actions
     */

    @FXML
    private void onAddCommandAction() {
        AddMultipleCommandDialog.showSimpleAddDialog(addCommandButton.getScene().getWindow(),
                getActiveExperimentViewController().getControllerUi().getController(),
                getActiveExperimentViewController().getControllerUi().getModelRspec().getNodes());

    }

    @FXML
    private void onAddBarrierAction() {
        getActiveExperimentViewController().getControllerUi().getController().addBarrierSegment();
    }

    /*
       Controller Viewer actions
     */

    @FXML
    private void onStartControllerAction() {
        getActiveExperimentViewController().getControllerUi().getController().startTime();
    }

    @FXML
    private void onPauseControllerAction() {
        getActiveExperimentViewController().getControllerUi().getController().stopTime();
    }

    @FXML
    private void onStopControllerAction() {
        getActiveExperimentViewController().getControllerUi().getController().stopTime();
        getActiveExperimentViewController().getControllerUi().getController().rewindTime();
    }


//    @FXML
//    private void onRewindControllerAction() {
//        getActiveExperimentViewController().getExperimentController().getController().rewindTime();
//    }

    @FXML
    private void onFastForwardControllerAction() {
        FastForwardDialog.showSimpleFastForwardDialog(
                getActiveExperimentViewController().getControllerUi().getController(),
                new Timestamp(getActiveExperimentViewController().getControllerUi().getModel().getElapsedTime()));

    }

    @FXML
    private void onInstantCommandAction() {
        InstantCommandDialog.showSimpleInstantCommandDialog(
                getActiveExperimentViewController().getControllerUi().getController(),
                getActiveExperimentViewController().getControllerUi().getModelRspec().getNodes());

    }

    @FXML
    private void onSaveExperimentResultsAction() {
        getActiveExperimentViewController().getControllerUi().getController().saveOutput();
    }
}
