package be.iminds.ilabt.jfed.experimenter_gui.slice.ribbon_tabs;

import be.iminds.ilabt.jfed.experimenter_gui.dialogs.DialogsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.actions.ExperimentActionsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.tasks.ExperimentTasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonButton;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;

import java.io.IOException;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 17:31
 */
public class RspecViewerRibbonTab extends SliceRibbonTab {
    private static final String RSPEC_VIEWER_RIBBON_TAB_FXML = "RspecViewerRibbonTab.fxml";

    private final ExperimentActionsFactory experimentActionsFactory;

    @FXML
    private RibbonButton saveExperimentButton;

    @FXML
    private RibbonButton saveNodeLoginInfoButton;

    public RspecViewerRibbonTab(DialogsFactory dialogsFactory, ExperimentActionsFactory experimentActionsFactory, ExperimentTasksFactory experimentTasksFactory) {
        this.experimentActionsFactory = experimentActionsFactory;
        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(RSPEC_VIEWER_RIBBON_TAB_FXML));

        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);
        fxmlLoader.setControllerFactory(aClass -> {
            if (aClass == GenericSliceRibbonComponentGroupController.class) {
                return new GenericSliceRibbonComponentGroupController(RspecViewerRibbonTab.this, experimentActionsFactory, experimentTasksFactory, dialogsFactory);
            } else if (aClass == AdvancedSliceRibbonComponentGroupController.class) {
                return new AdvancedSliceRibbonComponentGroupController(RspecViewerRibbonTab.this, dialogsFactory);
            }
            throw new RuntimeException("Unexpected class in ControllerFactory: " + aClass.getName());
        });


        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }

        setOnSelectionChanged(event -> {
            if (isSelected() && !getActiveExperimentViewController().isRawRspecVisible()) {
                getActiveExperimentViewController().switchToRawView();
            }
        });

    }

    @FXML
    public void onSaveExperimentAction() {
        experimentActionsFactory.createSaveManifestAction(getActiveExperimentViewController(), false /*saveToCSV*/).run();
    }

    @FXML
    public void onSaveNodeLoginInfoAction() {
        experimentActionsFactory.createSaveManifestAction(getActiveExperimentViewController(), true /*saveToCSV*/).run();
    }
}
