package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;

/**
 * User: twalcari
 * Date: 2/3/2015
 * Time: 18:09
 */
public class RebootSliverTask extends SingleCallExperimentTask {

    public static final int INITIAL_WAIT_MILLIS = 10_000;
    private final ExperimentController experimentController;
    private final ExperimentTasksFactory experimentTasksFactory;

    private final Task rebootTask;
    private final SfaAuthority authority;

    RebootSliverTask(ExperimentController experimentController, Sliver sliver, HighLevelController hlc, ExperimentTasksFactory experimentTasksFactory) {
        super(experimentController.getExperiment(), hlc);
        this.experimentController = experimentController;
        this.experimentTasksFactory = experimentTasksFactory;

        this.authority = sliver.getAuthority();
        this.rebootTask = hlc.getRestartSliverTask(sliver);

        updateTitle("Reboot sliver " + sliver.getUrn().toString());
    }

    RebootSliverTask(ExperimentController experimentController, SfaAuthority authority, HighLevelController hlc, ExperimentTasksFactory experimentTasksFactory) {
        super(experimentController.getExperiment(), hlc);
        this.experimentController = experimentController;
        this.experimentTasksFactory = experimentTasksFactory;

        this.authority = authority;
        this.rebootTask = hlc.getRestartSliceTask(experiment.getSlice(), authority);

        updateTitle("Rebooting nodes on " + authority.getHrn());
    }


    @Override
    public Task createTask() {
        return rebootTask;
    }

    @Override
    public void onTaskFinished(TaskExecution taskExecution) {
        if (taskExecution.getState() == TaskExecution.TaskState.SUCCESS) {
            experimentController.submitExperimentTask(experimentTasksFactory.createStatusUntilReadyTask(experiment, authority, INITIAL_WAIT_MILLIS));
        }
    }
}
