package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecLink;

import javax.annotation.Nonnull;

/**
 * User: twalcari
 * Date: 3/26/2015
 * Time: 10:03
 */
public class ShareLanTask extends SingleCallExperimentTask {

    private final AuthorityListModel authorityListModel;
    private final AuthorityFinder authorityFinder;
    private final FXRspecLink link;
    private final String sharedLanName;

    /**
     * TRUE if the link must be shared
     * FALSE if it must be unshared
     */
    private final boolean share;

    /**
     * @param experimentController
     * @param link
     * @param sharedLanName
     * @param share                true if the link must be shared. false if it must be unshared
     * @param hlc
     * @param authorityListModel
     */
    ShareLanTask(ExperimentController experimentController, @Nonnull FXRspecLink link, @Nonnull String sharedLanName, boolean share, HighLevelController hlc, AuthorityListModel authorityListModel, AuthorityFinder authorityFinder) {
        super(experimentController.getExperiment(), hlc);
        this.link = link;

        assert (!sharedLanName.isEmpty());
        this.sharedLanName = sharedLanName;

        this.share = share;

        this.authorityListModel = authorityListModel;
        this.authorityFinder = authorityFinder;

        updateTitle((share ? "Sharing" : "Unsharing")
                + " link " + link.getClientId() + " with ID '" + sharedLanName + "'");
    }

    @Override
    public Task createTask() {
        SfaAuthority auth = authorityFinder.findByUrn(link.getComponentManagerUrns().get(0), AuthorityFinder.Purpose.CREATE_SLIVER); //contact the same auth as would be contacted for CreateSliver

        if (share)
            return hlc.getShareLanTask(experiment.getSlice(), auth, link.getClientId(), sharedLanName);
        else
            return hlc.getUnshareLanTask(experiment.getSlice(), auth, link.getClientId(), sharedLanName);
    }
}
