package be.iminds.ilabt.jfed.experimenter_gui.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.ExperimentsTabPaneProvider;
import be.iminds.ilabt.jfed.experimenter_gui.dialogs.SelectAuthoritiesDialog;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ExperimentEditorTabFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewControllerFactory;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.collections.ObservableMap;
import javafx.stage.Window;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.inject.Singleton;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

/**
 * User: twalcari
 * Date: 12/1/2014
 * Time: 17:40
 */
@Singleton
public class TasksFactoryImpl implements TasksFactory {
    private final EasyModel easyModel;
    private final HighLevelController hlc;
    private final ExperimentViewControllerFactory experimentViewControllerFactory;
    private final ObservableMap<GeniUrn, ExperimentViewController> experimentViewControllers;
    private final Provider<SelectAuthoritiesDialog> selectAuthoritiesDialogProvider;
    private final ExperimentEditorTabFactory experimentEditorTabFactory;
    private final ExperimentsTabPaneProvider experimentsTabPaneProvider;
    private final AuthorityListModel authorityListModel;
    private final AuthorityFinder authorityFinder;
    private final JFedPreferences jFedPreferences;

    @Inject
    TasksFactoryImpl(EasyModel easyModel, TaskThread tt, HighLevelController hlc,
                     AuthorityListModel authorityListModel,
                     AuthorityFinder authorityFinder,
                     JFedPreferences jFedPreferences,
                     ExperimentViewControllerFactory experimentViewControllerFactory,
                     ObservableMap<GeniUrn, ExperimentViewController> experimentViewControllers,
                     Provider<SelectAuthoritiesDialog> selectAuthoritiesDialogProvider,
                     ExperimentEditorTabFactory experimentEditorTabFactory,
                     ExperimentsTabPaneProvider experimentsTabPaneProvider) {
        this.easyModel = easyModel;
        this.hlc = hlc;
        this.authorityListModel = authorityListModel;
        this.authorityFinder = authorityFinder;
        this.jFedPreferences = jFedPreferences;
        this.experimentViewControllerFactory = experimentViewControllerFactory;
        this.experimentViewControllers = experimentViewControllers;
        this.selectAuthoritiesDialogProvider = selectAuthoritiesDialogProvider;
        this.experimentEditorTabFactory = experimentEditorTabFactory;
        this.experimentsTabPaneProvider = experimentsTabPaneProvider;
    }

    @Override
    public RecoverSliceTask createRecoverSliceTask(String sliceUrn) {
        return new RecoverSliceTask(sliceUrn, easyModel, hlc, authorityListModel, authorityFinder, jFedPreferences, experimentViewControllerFactory, experimentViewControllers, selectAuthoritiesDialogProvider);

    }

    @Override
    public RecoverSliceTask createRecoverSliceTask(Slice slice) {
        return new RecoverSliceTask(slice, easyModel, hlc, authorityListModel, authorityFinder, jFedPreferences, experimentViewControllerFactory, experimentViewControllers, selectAuthoritiesDialogProvider);
    }

    @Override
    public DeleteDiskImageTask createDeleteDiskImageTask(SfaAuthority auth, AggregateManagerWrapper.ImageInfo imageInfo) {
        return new DeleteDiskImageTask(auth, imageInfo, hlc);
    }

    @Override
    public OpenRequestRspecTask createOpenRequestRspecTask(Window parentWindow, File file) throws FileNotFoundException{
        return new OpenRequestRspecTask(parentWindow, file, experimentEditorTabFactory, experimentsTabPaneProvider);

    }

    @Override
    public OpenRequestRspecTask createOpenRequestRspecTask(Window parentWindow, InputStream inputStream, long fileLength, String name, File defaultSaveFile){
        return new OpenRequestRspecTask(parentWindow,inputStream, fileLength,name,defaultSaveFile,experimentEditorTabFactory,experimentsTabPaneProvider);
    }

    @Override
    public OpenRequestRspecTask createOpenRequestRspecTask(Window parentWindow, String urlString) throws IOException {
        return new OpenRequestRspecTask(parentWindow, urlString, experimentEditorTabFactory, experimentsTabPaneProvider);
    }

}
