package be.iminds.ilabt.jfed.experimenter_gui.ui;

import be.iminds.ilabt.jfed.experimenter_gui.debug.DebugGuiFactory;
import com.google.inject.Injector;
import com.google.inject.grapher.graphviz.GraphvizGrapher;
import javafx.scene.Scene;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyCodeCombination;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * User: twalcari
 * Date: 12/16/2014
 * Time: 17:50
 */
public class AcceleratorService {
    private final Logger LOG = LoggerFactory.getLogger(AcceleratorService.class);

    private final DebugGuiFactory debugGuiFactory;
    private final GraphvizGrapher graphvizGrapher;
    private final Injector injector;

    @Inject
    public AcceleratorService(DebugGuiFactory debugGuiFactory, GraphvizGrapher graphvizGrapher, Injector injector) {
        this.debugGuiFactory = debugGuiFactory;
        this.graphvizGrapher = graphvizGrapher;
        this.injector = injector;
    }

    public void registerOnScene(Scene scene) {
        //add shortcut to debug-screen
        scene.getAccelerators().put(
                new KeyCodeCombination(KeyCode.F12, KeyCodeCombination.CONTROL_DOWN),
                debugGuiFactory::showDebugGui
        );
        scene.getAccelerators().put(
                new KeyCodeCombination(KeyCode.F11, KeyCodeCombination.CONTROL_DOWN),
                this::printInjectionGraph
        );
    }

    private void printInjectionGraph(){
        File file = new File("di-graph.dot");
        try {
            PrintWriter out = new PrintWriter(file, "UTF-8");

            graphvizGrapher.setOut(out);
            graphvizGrapher.setRankdir("TB");

            graphvizGrapher.graph(injector);
            LOG.info("Wrote Guice graph to {}", file.getAbsolutePath());
        } catch (IOException e) {
            LOG.error("Error while writing Guice graph to " + file.getAbsolutePath());
        }
    }
}
