package be.iminds.ilabt.jfed.experimenter_gui.util;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.editor.bo.NodeDescription;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import javafx.scene.image.Image;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.net.URL;

/**
 * User: twalcari
 * Date: 11/8/13
 * Time: 2:40 PM
 */
@Singleton
public class ImageUtil {

    private final JFedConfiguration config;


    @Inject
    public ImageUtil(JFedConfiguration config) {
        this.config = config;
    }


    public Image getRspecNodeImage(FXRspecNode node, ImageSize imageSize) {
        return getRspecNodeImage(config, node, imageSize);
    }

    public static Image getRspecNodeImage(JFedConfiguration config, FXRspecNode node, ImageSize imageSize) {
        NodeDescription nodeDescription = new NodeDescription("generic-node", null, null, null);

        if (node.getNodeDescriptionId() != null) {
            nodeDescription = config.getNodeDescription(node.getNodeDescriptionId());
        }

        return getNodeDescriptionImage(nodeDescription, imageSize);
    }

    private static final ImageSize[] FALLBACK_ORDER =
            {ImageSize.LARGE, ImageSize.MEDIUM, ImageSize.SMALL, ImageSize.X_SMALL};


    private static URL getNodeDescriptionImageURL(NodeDescription nodeDescription, ImageSize imageSize) {
        String urlString = String.format("/images/node_types/%d/%s.png",
                imageSize.getSize(), nodeDescription.getId());

        return ImageUtil.class.getResource(urlString);
    }

    public static Image getNodeDescriptionImage(NodeDescription nodeDescription, ImageSize imageSize) {
        URL url = getNodeDescriptionImageURL(nodeDescription, imageSize);

        int i = 0;
        while (url == null && i < FALLBACK_ORDER.length) {
            url = getNodeDescriptionImageURL(nodeDescription, FALLBACK_ORDER[i]);
            i++;
        }

        if (url != null)
            return new Image(url.toExternalForm());
        else
            return null;
    }

    private static URL getTestbedImageURL(JFedConfiguration.TestbedDescription testbedDescription, ImageSize imageSize) {
        String urlString = String.format("/images/testbeds/%d/%s",
                imageSize.getSize(), testbedDescription.getLogo());

        return ImageUtil.class.getResource(urlString);
    }

    public static Image getTestbedDescriptionImage(JFedConfiguration.TestbedDescription testbedDescription, ImageSize imageSize) {
        if (testbedDescription.getLogo() == null)
            return null;

        URL url = getTestbedImageURL(testbedDescription, imageSize);

        int i = 0;
        while (url == null && i < FALLBACK_ORDER.length) {
            url = getTestbedImageURL(testbedDescription, FALLBACK_ORDER[i]);
            i++;
        }

        if (url != null)
            return new Image(url.toExternalForm());
        else
            return null;
    }


}
