package be.iminds.ilabt.jfed.logging;

import ch.qos.logback.classic.spi.LoggingEvent;
import ch.qos.logback.core.AppenderBase;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * BugReportAppender
 */
public class BugReportAppender extends AppenderBase<LoggingEvent> {
    public static interface LoggingEventListener {
        public void onLoggingEvent(LoggingEvent e);
    }

    private final LinkedList<LoggingEvent> buffer;
    private int maxSize;
    private final List<LoggingEventListener> listeners;

    public BugReportAppender() {
        buffer = new LinkedList<>();
        listeners = new ArrayList<>();
        maxSize = 512;
    }

    @Override
    protected void append(LoggingEvent e) {
        buffer.add(e);
        if (maxSize >= 0) //so negative means infinite
            while (buffer.size() > maxSize)
                buffer.removeFirst();
        for (LoggingEventListener l : listeners) {
            l.onLoggingEvent(e);
        }
    }

    public int getMaxSize() {
        return maxSize;
    }
    public void setMaxSize(int maxSize) {
        this.maxSize = maxSize;
    }

    public synchronized List<LoggingEvent> getListCopy() {
        return new ArrayList<>(buffer);
    }
    public int getLength() {
        return buffer.size();
    }

    public void addListeners(LoggingEventListener l) {
        listeners.add(l);
    }
    public void removeListeners(LoggingEventListener l) {
        listeners.remove(l);
    }
}
