package be.iminds.ilabt.jfed.logging;

import be.iminds.ilabt.jfed.preferences.CorePreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.classic.joran.JoranConfigurator;
import ch.qos.logback.classic.util.ContextInitializer;
import ch.qos.logback.core.Appender;
import ch.qos.logback.core.joran.spi.JoranException;
import ch.qos.logback.core.util.StatusPrinter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.net.URL;

/**
 * JFedLoggerManager
 */
@Singleton
public class JFedLoggerManager {
    private static final Logger LOG = LoggerFactory.getLogger(JFedLoggerManager.class);

    private final JFedGuiPreferences preferences;

    @Inject
    public JFedLoggerManager(JFedGuiPreferences preferences) {
        this.preferences = preferences;
        updateLoggers();
    }

    private static final Level getLevel(final String levelName) {
        if (levelName == null)
            return null;
        if (levelName.equalsIgnoreCase("trace"))
            return Level.TRACE;
        if (levelName.equalsIgnoreCase("debug"))
            return Level.DEBUG;
        if (levelName.equalsIgnoreCase("info"))
            return Level.INFO;
        if (levelName.equalsIgnoreCase("warn"))
            return Level.WARN;
        if (levelName.equalsIgnoreCase("error"))
            return Level.ERROR;
        return null;
    }

    public void testLogging() {
        LOG.trace("TRACE test");
        LOG.debug("DEBUG test");
        LOG.info("INFO test");
        LOG.warn("WARN test");
        LOG.error("ERROR test");
    }

    //    public CyclicBufferAppender<LoggingEvent> getBugReportMemoryAppender() {
//        org.slf4j.Logger rootLogger = LoggerFactory.getLogger(org.slf4j.Logger.ROOT_LOGGER_NAME);
//        if (! (rootLogger instanceof ch.qos.logback.classic.Logger)) {
//            LOG.error("rootLogger is not instance of ch.qos.logback.classic.Logger but of " + rootLogger.getClass().getName());
//            return null;
//        }
//        ch.qos.logback.classic.Logger logbackRoot = (ch.qos.logback.classic.Logger) rootLogger;
//
//        Appender appender = logbackRoot.getAppender("BUGREPORT");
//        if (appender == null) {
//            LOG.error("BUGREPORT appender is not found");
//            return null;
//        }
//
//        if (!(appender instanceof CyclicBufferAppender)) {
//            LOG.error("BUGREPORT appender is not a CyclicBufferAppender but a "+appender.getClass().getName()+"");
//            return null;
//        }
//
//        CyclicBufferAppender<LoggingEvent> res = (CyclicBufferAppender<LoggingEvent>) appender;
//        return res;
//    }
    public BugReportAppender getBugReportMemoryAppender() {
        org.slf4j.Logger rootLogger = LoggerFactory.getLogger(org.slf4j.Logger.ROOT_LOGGER_NAME);
        if (! (rootLogger instanceof ch.qos.logback.classic.Logger)) {
            LOG.error("rootLogger is not instance of ch.qos.logback.classic.Logger but of " + rootLogger.getClass().getName());
            return null;
        }
        ch.qos.logback.classic.Logger logbackRoot = (ch.qos.logback.classic.Logger) rootLogger;

        Appender appender = logbackRoot.getAppender("BUGREPORT");
        if (appender == null) {
            LOG.error("BUGREPORT appender is not found");
            return null;
        }

        if (!(appender instanceof BugReportAppender)) {
            LOG.error("BUGREPORT appender is not a BugReportAppender but a "+appender.getClass().getName()+"");
            return null;
        }

        BugReportAppender res = (BugReportAppender) appender;
        return res;
    }

    public void updateLoggers() {
        if (!preferences.getBoolean(CorePreferenceKey.PREF_LOGGING_OVERWRITE)) {
            LOG.info("jFed is configured not to change the automatic logging configuration: Will reload default config.");
            LoggerContext loggerContext = (LoggerContext) LoggerFactory.getILoggerFactory();
            ContextInitializer ci = new ContextInitializer(loggerContext);
            URL url = ci.findURLOfDefaultConfigurationFile(true);

            try {
                JoranConfigurator configurator = new JoranConfigurator();
                configurator.setContext(loggerContext);
                loggerContext.reset();
                configurator.doConfigure(url);
            } catch (JoranException je) {
                // StatusPrinter will handle this
            }
            StatusPrinter.printInCaseOfErrorsOrWarnings(loggerContext);
            return;
        }

        LOG.info("jFed will update the logging configuration.");

        org.slf4j.Logger rootLogger = LoggerFactory.getLogger(org.slf4j.Logger.ROOT_LOGGER_NAME);
        if (! (rootLogger instanceof ch.qos.logback.classic.Logger)) {
            LOG.error("rootLogger is not instance of ch.qos.logback.classic.Logger but of "+rootLogger.getClass().getName());
            System.out.println("NOTE: logging cannot be modified by jFed: rootLogger is not instance of ch.qos.logback.classic.Logger but of "+rootLogger.getClass().getName());
            return;
        }
        ch.qos.logback.classic.Logger logbackRoot = (ch.qos.logback.classic.Logger) rootLogger;

        //loggers
        Level rootLevel = getLevel(preferences.getString(CorePreferenceKey.PREF_LOGGING_ROOT_LEVEL));
        Level terminalLevel = getLevel(preferences.getString(CorePreferenceKey.PREF_LOGGING_EXTTERM_LEVEL));

//        //appenders
//        Level consoleLevel = getLevel(preferences.getString(CorePreferenceKey.PREF_LOGGING_APPENDER_STDOUT_LEVEL));
//        Level bugreportLevel = getLevel(preferences.getString(CorePreferenceKey.PREF_LOGGING_APPENDER_BUGREPORT_LEVEL));

        if (rootLevel != null) {
            Level origLevel = logbackRoot.getLevel();
            if (origLevel.toInt() != rootLevel.toInt()) {
                LOG.info("jFed changes the root log level from "+origLevel.toString()+" to "+rootLevel.toString());
                logbackRoot.setLevel(rootLevel);
            } else
                LOG.info("jFed leaves the root log level unchanged at " + rootLevel);
        } else
            LOG.info("No root level specified in config: "+preferences.getString(CorePreferenceKey.PREF_LOGGING_ROOT_LEVEL));

        if (terminalLevel != null) {
            org.slf4j.Logger logger = LoggerFactory.getLogger("be.iminds.ilabt.jfed.ssh_terminal_tool");

            if (logger instanceof ch.qos.logback.classic.Logger) {
                ch.qos.logback.classic.Logger termLogger = (ch.qos.logback.classic.Logger) logger;
                Level origLevel = termLogger.getLevel();
                if (origLevel.toInt() != terminalLevel.toInt()) {
                    LOG.debug("Changing external terminal log level from "+origLevel+" to "+terminalLevel);
                    termLogger.setLevel(terminalLevel);
                } else
                    LOG.debug("Leaving external terminal log level unchanged at "+terminalLevel);
            } else {
                LOG.error("external terminal Logger is not instance of ch.qos.logback.classic.Logger but of " + logger.getClass().getName()+" -> Cannot be changed.");
            }
        }

        //console and bugreport level are currently not implemented (not useful enough)
//        if (consoleLevel != null) {
//            Appender appender = logbackRoot.getAppender("STDOUT");
//            if (appender == null) {
//                LOG.error("STDOUT appender is not found -> Cannot be changed.");
//            } else {
//                List<Filter> filterList = appender.getCopyOfAttachedFiltersList();
//                if (filterList == null || filterList.isEmpty() || (!(filterList.get(0) instanceof ch.qos.logback.classic.filter.ThresholdFilter))) {
//                    LOG.error("STDOUT appender cannot be changed because found filter list is: "+filterList);
//                } else {
//                    LOG.debug("Changing external terminal log level to " + consoleLevel+"   (unknown previous level)");
//                    ThresholdFilter filter = (ThresholdFilter) filterList.get(0);
//                    filter.setLevel(consoleLevel.toString());
//                }
//            }
//        }
//
//        if (bugreportLevel != null) {
//            Appender appender = logbackRoot.getAppender("BUGREPORT");
//            if (appender == null) {
//                LOG.error("BUGREPORT appender is not found -> Cannot be changed.");
//            } else {
//                List<Filter> filterList = appender.getCopyOfAttachedFiltersList();
//                if (filterList == null || filterList.isEmpty() || (!(filterList.get(0) instanceof ch.qos.logback.classic.filter.ThresholdFilter))) {
//                    LOG.error("BUGREPORT appender cannot be changed because found filter list is: "+filterList);
//                } else {
//                    LOG.debug("Changing external terminal log level to " + bugreportLevel+"   (unknown previous level)");
//                    ThresholdFilter filter = (ThresholdFilter) filterList.get(0);
//                    filter.setLevel(bugreportLevel.toString());
//                }
//            }
//        }
    }
}
