package be.iminds.ilabt.jfed.ssh_terminal_tool.standalone_test_app;

import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.ssh_terminal_tool.unix.SshAgentHelper;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.ui.javafx.userlogin.UserLoginFactory;
import be.iminds.ilabt.jfed.ui.javafx.util.ExtraInfoCallBackGUIs;
import be.iminds.ilabt.jfed.util.JFedUtils;
import com.cathive.fx.guice.GuiceApplication;
import com.cathive.fx.guice.GuiceFXMLLoader;
import com.google.inject.Module;
import javafx.application.Platform;
import javafx.scene.Scene;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import java.util.List;

/**
 * SshTerminalTestApp
 */
public class SshTerminalTestApp extends GuiceApplication {
    /* solution for MAC OS HeadlessException bug. See https://ibcn-jira.intec.ugent.be/browse/FEDIBBTDEV-239 */
    static {
        System.setProperty("java.awt.headless", "false");
    }

    private static final Logger LOG = LoggerFactory.getLogger(SshTerminalTestApp.class);
    private static final String SSH_TERMINAL_TEST_APP_FXML = "SshTerminalTestApp.fxml";

    @Inject
    private GuiceFXMLLoader loader;

    @Inject
    private UserLoginFactory userLoginFactory;
    @Inject
    private TaskThread taskThread;

    public static void main(String[] args) {
        launch(args);
    }

    @Override
    public void start(Stage stage) throws Exception {
        try {
            //check for filesystem access
            if (JFedUtils.getUserDataDirectory() == null) {

                JFDialogs.create()
                        .owner(stage)
                        .message("Please verify your security settings and try again.\n\n" +
                                "If you are trying to run jFed through Safari on OS X, you need to change your Safari security settings.\n" +
                                "(cfr. http://doc.fed4fire.eu/firstexperiment.html#start-up-jfed)")
                        .masthead("jFed was unable to access the filesystem.")
                        .title("Fatal Error").showError();

                exit();
                return;
            }

            ExtraInfoCallBackGUIs.prepareExtraInfoCallback(stage.getOwner());

            userLoginFactory.showUserLogin(true, null, successfulLogin -> {
                if (successfulLogin)
                    showMain();
                else
                    exit();
            });

            //prepare stage
            stage.setTitle("jFed Ssh Terminal Test App");

            stage.setOnCloseRequest(windowEvent -> exit());

            GuiceFXMLLoader.Result result = loader.load(SshTerminalTestApp.class.getResource(SSH_TERMINAL_TEST_APP_FXML));
            stage.setScene(new Scene(result.getRoot()));
            stage.showAndWait();


        } catch (Throwable ex) {
            LOG.error("Fatal error in jFed Ssh Terminal Test App Main Thread: " + ex.getMessage(), ex);
            JFDialogs.create()
                    .owner(stage)
                    .message("jFed Ssh Terminal Test App will terminate.")
                    .masthead("A fatal error occurred!")
                    .title("Fatal Error")
                    .showException(ex);

            throw ex;
        }
    }

    private void showMain() {
    }


    public void exit() {
        LOG.info("Close was requested. Trying to terminate");
        taskThread.requestStop();

        //stop any running ssh-agent
        SshAgentHelper.stopSshAgent();

        Platform.exit();
        System.exit(0);
    }

    @Override
    public void init(List<Module> modules) throws Exception {


    }
}
