package be.iminds.ilabt.jfed.lowlevel.connection;

import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.util.ExtraInfoCallback;
import be.iminds.ilabt.jfed.util.JFedTrustStore;

import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.List;

/**
 * ConnectionFactory
 */
public class BasicConnectionFactory implements ConnectionFactory {
    @Override
    public RestSslNoAuthConnection createRestSslNoAuthConnection(String serverUrlString, JFedTrustStore conTrustStore, JFedConnection.ProxyInfo usedProxyInfo, HandleUntrustedCallback handleUntrustedCallback, JFedConnection.DebugInfo debugInfo) throws JFedException {
        return new RestSslNoAuthConnection(
                serverUrlString, conTrustStore,
                usedProxyInfo,
                debugInfo, handleUntrustedCallback);
    }

    @Override
    public RestSslPasswordConnection createRestSslPasswordConnection(
            String serverUrlString, final JFedTrustStore conTrustStore, JFedConnection.ProxyInfo usedProxyInfo,
            ExtraInfoCallback.Login bonfireLogin,
            HandleUntrustedCallback handleUntrustedCallback, JFedConnection.DebugInfo debugInfo) throws JFedException {
        return new RestSslPasswordConnection(
                serverUrlString, conTrustStore,
                usedProxyInfo, bonfireLogin.getUsername(), bonfireLogin.getPassword(),
                debugInfo, handleUntrustedCallback);
    }

    @Override
    public SfaSslWithClientAuthConnection createSfaSslWithClientAuthConnection(
            String serverUrlStr, final JFedTrustStore conTrustStore, List<X509Certificate> clientCertChain, PrivateKey privateKey, JFedConnection.ProxyInfo usedProxyInfo,
            HandleUntrustedCallback handleUntrustedCallback, JFedConnection.DebugInfo debugInfo) throws JFedException {
        return new SfaSslWithClientAuthConnection(serverUrlStr, conTrustStore, clientCertChain,
                privateKey, usedProxyInfo, handleUntrustedCallback,
                debugInfo);
    }

    @Override
    public SfaSslWithoutClientAuthConnection createSfaSslWithoutClientAuthConnection(
            String serverUrlStr, final JFedTrustStore conTrustStore, JFedConnection.ProxyInfo usedProxyInfo,
            HandleUntrustedCallback handleUntrustedCallback, JFedConnection.DebugInfo debugInfo) throws JFedException {
        return new SfaSslWithoutClientAuthConnection(serverUrlStr, conTrustStore,
                usedProxyInfo, handleUntrustedCallback,debugInfo);
    }

    @Override
    public SfaPlainHttpConnection createSfaPlainHttpConnection(
            String serverUrlStr, JFedConnection.ProxyInfo usedProxyInfo,
            JFedConnection.DebugInfo debugInfo) throws JFedException {
        return new SfaPlainHttpConnection(serverUrlStr, usedProxyInfo, debugInfo);
    }
}
