package be.iminds.ilabt.jfed.lowlevel.connection;

import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.util.KeyUtil;
import be.iminds.ilabt.jfed.util.SshServerProxyHelper;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.CoreConnectionPNames;
import org.apache.http.params.HttpParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.net.URL;


/**
 * Non encrypted HTTP connection
 */
public class ClientPlainAuthenticationXmlRpcTransportFactory extends CommonsHttpClientXmlRpcTransportFactory {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(ClientPlainAuthenticationXmlRpcTransportFactory.class);

    /**
     * @param serverUrl URL of server to connect to
     */
    public ClientPlainAuthenticationXmlRpcTransportFactory(final String serverUrl, boolean debugMode, final JFedConnection.ProxyInfo proxyInfo) throws JFedException {
        super(serverUrl, getHttpClient(serverUrl, debugMode, proxyInfo), debugMode,
                new CommonsHttpClientXmlRpcTransportFactory.LogDebugInfoCallback() {
                    @Override
                    public void logDebugInfo(Logger log) {
                        log.info("ClientPlainAuthenticationXmlRpcTransportFactory debug info:\n" +
                                        "proxyInfo=" + proxyInfo + "\n" +
                                        "serverUrl=" + serverUrl
                        );
                    }
                }
        );
    }


    private static DefaultHttpClient getHttpClient(String serverUrlStr, boolean debugMode, JFedConnection.ProxyInfo proxyInfo) throws JFedException {
        if (serverUrlStr == null) throw new RuntimeException("serverUrlStr == null");

        final URL serverUrl;
        try {
            serverUrl = new URL(serverUrlStr);
        } catch (MalformedURLException e) {
            LOG.error("ERROR: MalformedURLException url=\"" + serverUrlStr + "\"", e);
            return null;
        }

        HttpParams params = new BasicHttpParams();
        params.setParameter(CoreConnectionPNames.SO_TIMEOUT, 120000); //read timeout in ms (default: 0 = infinite)   (long timeout, as some calls take time to finish)
        params.setParameter(CoreConnectionPNames.CONNECTION_TIMEOUT, 10000); //connection timeout in ms (default: 0 = infinite)


        DefaultHttpClient res = new DefaultHttpClient(params);

        if (proxyInfo != null && proxyInfo instanceof JFedConnection.SshProxyInfo) {
            JFedConnection.SshProxyInfo sshProxyInfo = (JFedConnection.SshProxyInfo) proxyInfo;

            LOG.debug("Using SSH Proxy for plain HTTP connection: " + sshProxyInfo.getHostname());

            SchemeSocketFactory socketFactory = new SshServerProxyHelper.PlainHttpOverSshProxySocketFactory(
                    new SshServerProxyHelper.SshProxyInfo(
                            new InetSocketAddress(sshProxyInfo.getHostname(), sshProxyInfo.getPort()),
                            sshProxyInfo.getUsername(),
                            sshProxyInfo.getHostKey(),
                            new String(KeyUtil.privateKeyToAnyPem(sshProxyInfo.getSshKeyInfo().getPrivateKey()))
                    )
            );

            Scheme sch1 = new Scheme("http", 80/*port is the default port for the given protocol*/, socketFactory);
            res.getConnectionManager().getSchemeRegistry().register(sch1);
        }

        return res;
    }
}
