package be.iminds.ilabt.jfed.lowlevel.connection;

import be.iminds.ilabt.jfed.lowlevel.JFedException;
import org.slf4j.Logger;

import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.List;


/**
 * This XmlRpcTransportFactory will read a PEM certificate with a client key and certificate,
 * add the needed trust certificates, and setup an SSL connection. This SSL connection will be used to for the XmlRpc
 * calls.
 */
public class ClientSslAuthenticationXmlRpcTransportFactory extends CommonsHttpClientXmlRpcTransportFactory {
    /**
     * @param proxyInfo
     * @param clientCertificateChain chain of X509Certificates to use for the SSL connection
     * @param privateKey             PrivateKey to use for client authentication of the SSL connection
     * @param serverUrl              URL of server to connect to
     * @param trustStore             The trustStore with trusted roots and trusted self signed certificates, that will be used for the SSL connection
     */
    public ClientSslAuthenticationXmlRpcTransportFactory(final JFedConnection.ProxyInfo proxyInfo,
                                                         final List<X509Certificate> clientCertificateChain,
                                                         final PrivateKey privateKey,
                                                         final String serverUrl,
                                                         final KeyStore trustStore,
                                                         final Collection<String> allowedCertificateHostnameAliases,
                                                         boolean debugMode,
                                                         HandleUntrustedCallback handleUntrustedCallback) throws JFedException {
        super(serverUrl, HttpsClientWithUserAuthenticationFactory.getHttpClient(proxyInfo, clientCertificateChain, privateKey, serverUrl,
                        trustStore, allowedCertificateHostnameAliases, debugMode, handleUntrustedCallback), debugMode,
                new CommonsHttpClientXmlRpcTransportFactory.LogDebugInfoCallback() {
                    @Override
                    public void logDebugInfo(Logger log) {
                        String trustStoreAliases = null;
                        try {
                            if (trustStore != null && trustStore.aliases() != null)
                                trustStoreAliases = trustStore.aliases().toString();
                        } catch (KeyStoreException e) {
                            trustStoreAliases = "Exception: " + e.getMessage();
                        }

                        log.info("ClientSslAuthenticationXmlRpcTransportFactory debug info:\n" +
                                        "proxyInfo=" + proxyInfo + "\n" +
                                        "clientCertificateChain=" + clientCertificateChain + "\n" +
                                        "privateKey=" + (privateKey == null ? null : "not null") + "\n" +
                                        "serverUrl=" + serverUrl + "\n" +
                                        "allowedCertificateHostnameAliases=" + allowedCertificateHostnameAliases + "\n" +
                                        "trustStore.aliases=" + trustStoreAliases + "\n"
                        );
                    }
                }
        );
    }
}
