package be.iminds.ilabt.jfed.lowlevel.connection;

import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import org.apache.xmlrpc.XmlRpcClient;

import java.net.URL;

/**
 * GeniConnection: XmlRpc over HTTP without SSL (= unencrypted, plain HTTP)
 */
public class SfaPlainHttpConnection extends SfaConnection {
    private CommonsHttpClientXmlRpcTransportFactory xmlRpcTransportFactory;
    private XmlRpcClient xmlRpcClient;

    /**
     * Create a connection with debugMode disabled and with the specified auth added as debugInfo
     *
     * @param serverUrl url to connect to (may not be null)
     * @param auth the authority to add for debug information purposes (may be null)
     * @param proxyInfo the proxy to use (may be null if no proxy)
     * */
    public SfaPlainHttpConnection(String serverUrl, SfaAuthority auth, ProxyInfo proxyInfo) throws JFedException {
        this(serverUrl, proxyInfo, new DebugInfo(false, auth, null));
    }

    /**
     * Create a connection by specifying all low level details needed.
     *
     * @param serverUrl url to connect to (may not be null, unless this is a fake connection used for debugging)
     * @param proxyInfo the proxy to use (may be null if no proxy)
     * @param debugInfo debug information, such as authority and user, and whether debug mode is enabled. (may be null)
     * @throws JFedException
     */
    public SfaPlainHttpConnection(String serverUrl, ProxyInfo proxyInfo, DebugInfo debugInfo) throws JFedException {
        super(serverUrl, null, proxyInfo, debugInfo);

        xmlRpcTransportFactory = new ClientPlainAuthenticationXmlRpcTransportFactory(serverUrl, this.debugInfo.isDebugMode(), proxyInfo);
        try {
            xmlRpcClient = new XmlRpcClient(new URL(serverUrl), xmlRpcTransportFactory);
            error = false;
        } catch (Exception e) {
            throw new JFedException("Error creating XmlRpcClient in GeniConnection constructor: " + e.getMessage(), e);
        }
    }

    @Override
    public CommonsHttpClientXmlRpcTransportFactory getXmlRpcTransportFactory() {
        assert !debugInfo.isFakeForDebugging() :
                "Error: This connection is a fake debugging connection, yet the CommonsHttpClientXmlRpcTransportFactory was requested";
        return xmlRpcTransportFactory;
    }

    @Override
    public XmlRpcClient getXmlRpcClient() {
        assert !debugInfo.isFakeForDebugging() :
                "Error: This connection is a fake debugging connection, yet the XmlRpcClient was requested";
        return xmlRpcClient;
    }

    public void clearLastCallData() {
        xmlRpcTransportFactory.clearLastCallData();
    }

    public void close() {
        error = true;
        xmlRpcTransportFactory.clearLastCallData();
        xmlRpcTransportFactory.close();
        xmlRpcTransportFactory = null;
        xmlRpcClient = null;
    }
}